import { Image, View } from '@tarojs/components'
import Taro from '@tarojs/taro'
import { useEffect, useMemo, useState } from 'react'
import styles from './index.module.scss'
import { formatImgUrl } from '@/common/format'

export interface colorParams {
  value?: {
    texture_url?: string // 纹理图路径
    lab?: { l: number; a: number; b: number } // lab
    rgb?: { r: number; g: number; b: number } // rgb
    title?: string // 标题
  }
  show?: boolean
  onClose?: () => void
  showNumber?: number // 图片显示张数，0不限制
  suffix?: string
}
const LabAndImgShow = (props: colorParams) => {
  const { value, show = false, onClose, showNumber = 1, suffix = '!w200' } = props
  // 显示颜色
  const [labShow, setLabShow] = useState(false)
  // lab是否都是0
  const rgbStyle = useMemo(() => {
    if (value?.lab && (value.lab.l || value.lab.a || value.lab.b)) {
      return { backgroundColor: `rgb(${value.rgb?.r} ${value.rgb?.g} ${value.rgb?.b})` }
    }
    else {
      return null
    }
  }, [value])

  // 显示图片
  const onShowImage = () => {
    onClose?.()
    let res: string[] = []
    if (value?.texture_url) {
      res = value?.texture_url?.split(',').map(item => formatImgUrl(item, suffix))
    }
    console.log('res==>', res)
    const n_res = showNumber == 0 ? res : res?.splice(0, showNumber)
    console.log('第一张图片', n_res[0])
    Taro.previewImage({
      current: n_res[0],
      urls: n_res,
    })
  }
  const [reload, setReload] = useState(false)
  // 检测图片是否加载成功
  const checkLoad = () => {
    console.log('图片加载失败')
    setReload(true)
  }

  const onLoad = () => {
    setReload(false)
  }
  useEffect(() => {
    if (reload) {
      if (show && rgbStyle) {
        setLabShow(true)
        return
      }
    }
    if (show && value?.texture_url) {
      onShowImage()
    }
    else if (show && rgbStyle) {
      setLabShow(true)
    }
  }, [show, reload])

  useEffect(() => {
    if (!show) { setLabShow(false) }
  }, [show])
  return (
    <View>
      {labShow && (
        <View className={styles.main} catch-move="true" onClick={() => onClose?.()}>
          <View className={styles.con}>
            <View className={styles.rgb} style={rgbStyle!}></View>
            <View className={styles.name}>{value?.title}</View>
          </View>
        </View>
      )}
      {
        value?.texture_url && <Image
          src={formatImgUrl(value?.texture_url)}
          onError={checkLoad}
          onLoad={onLoad}
          style={{ position: 'absolute', top: '9999', width: '0', height: '0' }}
        ></Image>
      }

    </View>
  )
}

export default LabAndImgShow
