import { View } from '@tarojs/components'
import { FC, forwardRef, memo, useEffect, useImperativeHandle, useMemo, useRef, useState } from 'react'
import DropDownItem from '../dropDown-item'
import FilterButton from '../filterButton'
import styles from './index.module.scss'
import { EnumMarketingDepartmentApi } from '@/api/index'
import { EnumSaleMode } from '@/common/Enumerate'
import { selectList } from '@/common/Enumerate/BaseEnum'

type ChangedValue = string | number

interface SelectSaleTypeProps {
  onChange?: (value: ChangedValue) => void
  onCloseOverlay?: () => void
}

interface EnumList {
  id: number
  code: string
  name: string
}
const manual = [
  { id: EnumSaleMode.Bulk, code: '0', name: selectList[EnumSaleMode.Bulk].title },
  { id: EnumSaleMode.Plate, code: '1', name: selectList[EnumSaleMode.Plate].title },
  { id: EnumSaleMode.BulkCut, code: '2', name: selectList[EnumSaleMode.BulkCut].title },
]
// 订单类型
const SelectOrderType = (props: SelectSaleTypeProps, ref) => {
  const selectName = '订单类型'

  const { onChange, onCloseOverlay } = props
  console.log(props)
  const { fetchData } = EnumMarketingDepartmentApi()

  const [enumList, setEnumList] = useState<EnumList[]>([])

  const getData = async() => {
    // const res = await fetchData()
    setEnumList([{ id: -1, code: '', name: '全部' }, ...manual])
  }

  useEffect(() => {
    getData()
  }, [])

  const [currentValue, setCurrentValue] = useState<ChangedValue>(-1)

  const handleClick = (value: ChangedValue) => {
    setCurrentValue(value)
    onChange?.(value)
  }
  const dropDownRef = useRef<any>(null)

  useImperativeHandle(
    ref,
    () => {
      return {
        show: dropDownRef.current.show,
        showPopup: dropDownRef.current.showPopup,
        closePopup: dropDownRef.current.closePopup,
      }
    },
    [dropDownRef.current],
  )

  const displayTitle = useMemo(() => {
    if (currentValue === -1) {
      return selectName
    }
    return enumList.length ? enumList.filter(option => option.id === currentValue)?.[0]?.name : selectName
  }, [enumList, currentValue])

  return (
    <DropDownItem title={displayTitle} value={currentValue} activeColor="#337fff" ref={dropDownRef} onCloseOverlay={onCloseOverlay}>
      <View className={styles.grid}>
        {!!enumList.length
          && enumList.map((item: EnumList, key) => {
            return (
              <FilterButton circle key={key} isActive={item.id === currentValue} onClick={() => handleClick(item.id)}>
                {item.name}
              </FilterButton>
            )
          })}
      </View>
    </DropDownItem>
  )
}
export default memo(forwardRef(SelectOrderType))
