/* eslint-disable @typescript-eslint/indent */
import type { CommonEventFunction, InputProps } from '@tarojs/components'
import { Image, Input, ScrollView, Text, View } from '@tarojs/components'
import { memo, useCallback, useEffect, useMemo, useRef, useState } from 'react'
import classnames from 'classnames'
import styles from './index.module.scss'
import LabAndImg from '@/components/LabAndImg'
import { formatPriceDiv, formatWeightDiv } from '@/common/format'
import { EnumSaleMode } from '@/common/Enumerate'
import { isInteger } from '@/common/common'
import { selectList } from '@/common/Enumerate/BaseEnum'
import Counter from '@/components/counter'
import IconFont from '@/components/iconfont/iconfont'

interface props {
  clickAdd: (any) => void
  handleSubtract: (any) => void
  handPlus: (any) => void
  modeFont: EnumSaleMode
  value: {
    showInput?: boolean
    nums?: number
    bulk_price?: number
    length_cut_price?: number
    weight_cut_price?: number
    code?: string
    rgb?: { r: number; g: number; b: number }
    lab?: { l: number; a: number; b: number }
    texture_url: string
    name?: string
    roll?: number | string
    buyNums?: number | string
    id?: number
    weight_error?: number | string
    last_bulk_price?: number
    last_length_cut_price?: number
    last_weight_cut_price?: number
  }
  onBlur?: (a: any) => void
}
const GoodsItem = (props: props) => {
  const {
    clickAdd,
    handleSubtract,
    handPlus,
    modeFont,
    value: valueInfo,
    onBlur,
  } = props

  const [showInput, setShowInput] = useState(false)

  const defaultValue = useMemo(() => {
    console.log('modeFont', modeFont)
    return selectList[modeFont || 0].defaultNum
  }, [modeFont])

  const [value, setValue] = useState<any>(defaultValue)

  console.log('render GoodsItem', valueInfo)

  const onInputEven = (e) => {
    // const res = Number(e)
    // if (res < defaultValue) {
    //   setValue(defaultValue)
    // }
    // else {
    //   setValue(res)
    // }
      setValue(e)
    // if (modeFont === EnumSaleMode.Bulk) {
    //   if (Number(res) < defaultValue) {
    //     setValue(defaultValue)
    //   }
    // }
    // if (modeFont === EnumSaleMode.Plate) {
    //   if (Number(res) < defaultValue) {
    //     setValue(defaultValue)
    //   }
    // }
    // if (modeFont === EnumSaleMode.BulkCut) {
    //   if (Number(res) < defaultValue) {
    //     setValue(defaultValue)
    //   }
    // }
  }

  const labAndImgObj = (item) => {
    return { lab: item.lab, rgb: item.rgb, texture_url: item.texture_url }
  }

  const handleClickAdd = () => {
    console.log('handleClickAdd')
    setShowInput(true)
    clickAdd?.({ ...valueInfo, nums: defaultValue })
  }

  const clickSubtract = () => {
    let tempValue = value
    const payload = {
      checked: showInput,
      ...valueInfo,
      nums: value,
    }
    console.log(value)
    if (Math.trunc(defaultValue + 1) >= value && value > defaultValue) {
      tempValue = defaultValue
    }
    else {
      tempValue--
    }
    if (tempValue < defaultValue) {
      setShowInput(false)
      payload.checked = false
      tempValue = defaultValue
      setValue(tempValue)
      payload.nums = tempValue
    }
    else {
      payload.nums = tempValue
      setValue(payload.nums)
    }
    handleSubtract?.(payload)
  }

  const clickPlus = () => {
    let tempValue = value
    console.log('clickPlus', tempValue)
    const payload = {
      checked: showInput,
      ...valueInfo,
      nums: value,
    }
    tempValue++
    payload.nums = Math.trunc(tempValue)
    setValue(payload.nums)
    handPlus?.(payload)
  }

  const handleBlur = (e) => {
    const payload = {
      checked: showInput,
      ...valueInfo,
      nums: value,
    }
    const res = Number(e)
    if (modeFont === EnumSaleMode.Bulk) {
      if ((e == '' || res === defaultValue)) {
        setShowInput(false)
        payload.checked = false
        payload.nums = defaultValue
      }
      else if ((e != '' || res > defaultValue)) {
        payload.nums = Math.trunc(e)
      }
    }
    if (modeFont === EnumSaleMode.Plate) {
      if (res < defaultValue) {
        payload.nums = defaultValue
        setShowInput(false)
        payload.checked = false
      }
      else if (res >= defaultValue) {
        if (isInteger(res)) {
          payload.nums = res
        }
        else {
          payload.nums = res.toFixed(2)
        }
      }
    }
    if (modeFont === EnumSaleMode.BulkCut) {
      if (res < defaultValue) {
        payload.nums = defaultValue
        setShowInput(false)
        payload.checked = false
      }
      else if (res >= defaultValue) {
        if (isInteger(res)) {
          payload.nums = res
        }
        else {
          payload.nums = res.toFixed(2)
        }
      }
    }
    setValue(Number(payload.nums))
    onBlur?.(payload)
  }
  // 格式化单位
  const formatUnit = (sale_mode: EnumSaleMode) => {
    return sale_mode == EnumSaleMode.Bulk ? '条' : '米'
  }
  const onClickBtn = (num) => {
    if (value > num || num === defaultValue) {
      clickSubtract()
    }
    else if (value < num) {
      clickPlus()
    }
  }
  return (
    <View className={styles.goodsBox}>
      <View className={styles.pic}>
        <LabAndImg suffix="!w200" value={labAndImgObj(valueInfo)} />
      </View>
      <View className={styles.rightGoodbox}>
        <View className={styles.leftFontsbox}>
          <View className={styles.title}>{valueInfo?.code} {valueInfo?.name}</View>
          {/* <View className={styles.productNums}>剩：{valueInfo?.roll}件</View> */}
          {
            modeFont === EnumSaleMode.Bulk && <View className={styles.productName}>空差：{formatWeightDiv(valueInfo?.weight_error) || 0}Kg</View>
          }
        </View>
        <View className={styles.rightFontsbox}>
          <View className={styles.moneyFlex}>
            {
              (modeFont === EnumSaleMode.Bulk) && (valueInfo?.last_bulk_price !== valueInfo?.bulk_price)
                ? <View className={styles.lastBox}>
                ¥{formatPriceDiv(valueInfo?.last_bulk_price)} /Kg
                </View>
                : null
            }
            <View className={styles.money}>¥{modeFont === EnumSaleMode.Bulk ? formatPriceDiv(valueInfo?.bulk_price) : modeFont === EnumSaleMode.Plate ? formatPriceDiv(valueInfo?.length_cut_price) : formatPriceDiv(valueInfo?.weight_cut_price)}{modeFont !== EnumSaleMode.Plate ? '/Kg' : '/m'}</View>
          </View>
          {
            showInput
            // <View className={styles.inputBox}>
            //   <View className={styles.reduceBox} onClick={clickSubtract}>
            //     <View className={styles.reduce}></View>
            //   </View>
            //   <View className={styles.numsBox}>
            //     <Input alwaysEmbed cursorSpacing={100} adjustPosition type={type as any} value={value} onInput={onInputEven} onBlur={handleBlur}></Input>
            //   </View>
            //   <View className={styles.addBox} onClick={clickPlus}>+</View>
            // </View>
              ? <Counter
                  onBlue={handleBlur}
                  defaultNum={value}
                  onChange={onInputEven}
                  step={selectList[modeFont].step}
                  digits={selectList[modeFont].digits}
                  onClickBtn={onClickBtn}
                  unit={formatUnit(modeFont)}
                  minNum={selectList[modeFont].minNum}
                  maxNum={selectList[modeFont].maxNum}
              />
              : <View className={styles.addtine} onClick={handleClickAdd}>
                <IconFont name="icon-xinzenganniu" size={45}></IconFont>
              </View>
          }
        </View>
      </View>
    </View>
  )
}
export default memo(GoodsItem)
