import { Text, View } from '@tarojs/components'
import Taro, { usePullDownRefresh, useRouter } from '@tarojs/taro'
import type { FC } from 'react'
import { useCallback, useLayoutEffect, useMemo, useState } from 'react'
import styles from './index.module.scss'
import { DeliverNoticeOrder, DeliveryNoticeOrderAudit, DeliveryNoticeOrderUpload } from '@/api'
import { formatDateTime, formatMeterDiv, formatWeightDiv } from '@/common/format'
import Cell from '@/components/cell'
import Divider from '@/components/Divider'
import LayoutBlock from '@/components/layoutBlock'
import NormalButton from '@/components/normalButton'
import SaleModeTag from '@/components/saleModeTag'
import UploadImage from '@/components/uploadImage'
import { alert } from '@/common/common'
import { TakeOutOfStorage } from '@/api/delivery'
import { EnumSaleMode } from '@/common/Enumerate'

const DeliveryDetail: FC = () => {
  usePullDownRefresh(() => {
    getData()
  })

  const router = useRouter()

  const { fetchData } = DeliverNoticeOrder()
  const { fetchData: takeOut } = TakeOutOfStorage()

  const [detailInfo, setDetailInfo] = useState<Record<string, any>>({})

  // const [isPending, startTransition] = useTransition()

  const getData = async() => {
    const { id } = router.params
    const res = await fetchData({
      id,
    })
    console.log('res==>', res.data)
    // startTransition(() => {
    setDetailInfo(res.data)
    // })
    Taro.stopPullDownRefresh()
  }

  useLayoutEffect(() => {
    getData()
  }, [])

  const { fetchData: uploadData } = DeliveryNoticeOrderUpload()

  const { fetchData: FetchAudit } = DeliveryNoticeOrderAudit()

  // const DetailItem = useMemo(() => {
  //   const element
  //     = !!detailInfo?.weight_list?.length
  //     && detailInfo?.weight_list.map((weightItem, index) => {
  //       return (
  //         <View key={index} className={styles.detailInfo}>
  //           <Divider direction="horizontal" customClassName={styles.divider}></Divider>
  //           <View className={styles.detailInfoItem}>
  //             <View className={styles['detailInfoItem--title']}>
  //               <View className="">
  //                 <Text style={{ marginRight: '20rpx' }}>
  //                   {weightItem.product_code} {weightItem.product_name}
  //                 </Text>
  //                 <SaleModeTag saleMode={weightItem?.sale_mode} />
  //               </View>
  //               <View className="">共 {weightItem?.sale_mode === 0 ? `${weightItem.roll} 条` : `${formatMeterDiv(weightItem?.length || 0)} 米`}</View>
  //             </View>
  //             <View className={styles['detailInfoItem--detail']}>
  //               <View className={styles['detailInfoItem--detail--row']}>
  //                 <View className={styles['detailInfoItem--detail--name']}>
  //                   {weightItem.product_color_code} {weightItem.product_color_name}
  //                 </View>
  //                 <View className={styles['detailInfoItem--detail--count']}>
  //                   x{weightItem?.sale_mode === 0 ? `${weightItem.roll} 条` : `${formatMeterDiv(weightItem?.length || 0)} 米`}
  //                 </View>
  //                 <View className={styles['detailInfoItem--detail--weight']}>{formatWeightDiv(weightItem.weight)}kg</View>
  //               </View>
  //             </View>
  //           </View>
  //         </View>
  //       )
  //     })
  //   return <>{element}</>
  // }, [detailInfo])

  const [readyToUploadList, setReadyToUploadList] = useState<string[]>([])

  const handleUploadChange = (imageList: string[]) => {
    console.log('imageList===>', imageList)
    setReadyToUploadList([...imageList])
  }
  // 上传附件
  const handleUploadPic = useCallback(async() => {
    if (!readyToUploadList.length) {
      return Promise.reject(new Error('请先上传附件'))
    }
    console.log('readyToUploadList:::', readyToUploadList)
    const res = await uploadData({
      id: Number(router.params.id),
      delivery_appendix: readyToUploadList,
    })
    if (res.success) {
      // alert.success('上传成功')
      // getData()
      return Promise.resolve()
    }
    else {
      return Promise.reject(new Error('上传失败'))
    }
  }, [readyToUploadList])

  // 审核
  const handleAudit = useCallback(() => {
    // if (detailInfo?.delivery_appendix_url?.length === 0) {
    //   alert.error('请先上传附件')
    //   return
    // }
    Taro.showModal({
      confirmColor: '#337FFF',
      title: '确定审核？',
      async success(res) {
        if (res.confirm) {
          try {
            await handleUploadPic()
          }
          catch (e) {
            e.name = ''
            console.log('e', e.toString())
            alert.error(e.toString())
            return
          }
          const res = await FetchAudit({ id: Number(router.params.id) })
          if (res.success) {
            alert.success('审核成功')
            getData()
          }
          else {
            alert.none(res.msg)
          }
        }
        else if (res.cancel) {
          console.log('用户点击取消')
        }
      },
    })
  }, [readyToUploadList])

  const handleTakeOutOfStorage = async() => {
    const res = await takeOut({
      sale_order_id: detailInfo.sale_order_id,
    })
    if (res.success) {
      Taro.showToast({ title: '出仓成功', icon: 'success' })
      getData()
    }
    else {
      Taro.showToast({ title: '出仓失败', icon: 'error' })
    }
  }

  const BottomBar = () => {
    return (
      <View className={styles.bottomBar} style={!detailInfo?.is_outbound ? { justifyContent: 'space-between' } : { justifyContent: 'center' }}>
        {
          !detailInfo?.is_outbound && <NormalButton
            customClassName={styles.bottomBar__button}
            plain
            type="info"
            customStyles={{ width: '45%', color: '#636363', borderColor: '#c8c8c8' }}
            round
            onClick={handleTakeOutOfStorage}
          >
          确认出仓
          </NormalButton>
        }
        <NormalButton type="primary" round customStyles={{ width: !detailInfo?.is_outbound ? '45%' : '90%' }} onClick={handleAudit}>
          确认送达
        </NormalButton>
      </View>
    )
  }

  const groups = useMemo(() => {
    const temp: Record<string, any> = {}
    detailInfo?.weight_list?.forEach((weightItem) => {
      const name = `${weightItem.product_code} ${weightItem.product_name}`
      temp.hasOwnProperty(name) ? temp[name].push(weightItem) : temp[name] = [weightItem]
    })
    return temp
  }, [detailInfo])

  const formatUnit = (saleMode: EnumSaleMode) => {
    return saleMode === EnumSaleMode.Bulk ? '条' : '米'
  }

  return (
    <View className={styles.deliveryDetail}>
      <View className={styles.content}>
        <LayoutBlock circle customStyle={{ padding: '24rpx' }}>
          <View className={styles.detailTop}>
            <View className={styles.orderNo}>发货单号：{detailInfo?.order_no}</View>
            {detailInfo?.status === 0 ? <View className={styles['status--toBeAudit']}>待发货</View> : <View className={styles['status--audited']}>已完成</View>}
          </View>
          {
            Object.entries(groups).map(([name, group], index) => {
              let total = 0
              if (group[0]?.sale_mode === EnumSaleMode.Bulk) {
                total = group.reduce((acc, cur) => acc + cur.roll, 0)
              }
              else {
                total = group.reduce((acc, cur) => acc + formatMeterDiv(cur.length), 0)
              }
              return (
                <View key={index} className={styles.detailInfo}>
                  <Divider direction="horizontal" customClassName={styles.divider}></Divider>
                  <View className={styles.detailInfoItem}>
                    <View className={styles['detailInfoItem--title']}>
                      <View className="">
                        <Text style={{ marginRight: '20rpx' }}>
                          {name}
                        </Text>
                        <SaleModeTag saleMode={group[0]?.sale_mode} />
                      </View>
                      <View className="">共 {total} {formatUnit(group[0]?.sale_mode)}</View>
                    </View>
                    <View className={styles['detailInfoItem--detail']}>
                      {
                        group?.map((weightItem, key) => {
                          return (
                            <View className={styles['detailInfoItem--detail--row']} key={key}>
                              <View className={styles['detailInfoItem--detail--name']}>
                                {weightItem.product_color_code} {weightItem.product_color_name}
                              </View>
                              <View className={styles['detailInfoItem--detail--count']}>
                                    x{weightItem?.sale_mode === 0 ? `${weightItem.roll} 条` : `${formatMeterDiv(weightItem?.length || 0)} 米`}
                              </View>
                              <View className={styles['detailInfoItem--detail--weight']}>{formatWeightDiv(weightItem.weight)}kg</View>
                            </View>
                          )
                        })
                      }
                    </View>
                  </View>
                </View>
              )
            })
          }
          <View className={styles.total}>
            <Text>汇总：</Text>
            <Text className={styles.totalContent}>
              共{detailInfo?.delivery_product_nums || 0}种面料，{detailInfo?.delivery_product_color_nums || 0}种颜色，共
              {detailInfo?.sale_mode === 0 ? `${detailInfo?.total_roll}条` : `${formatMeterDiv(detailInfo?.total_length).toLocaleString()}米`}
              ，重量{formatWeightDiv(detailInfo?.total_weight).toLocaleString()}kg
            </Text>
          </View>
        </LayoutBlock>
        <LayoutBlock circle customStyle={{ padding: '24rpx' }}>
          <View className={styles.orderInfoTop}>订单信息</View>
          <Divider direction="horizontal" customClassName={styles.divider}></Divider>
          <View className="orderInfoDetail">
            <Cell title="订单备注：" desc={detailInfo?.sale_order_remark || '暂无备注信息'}></Cell>
            <Cell title="创建时间：" desc={formatDateTime(detailInfo?.create_time) || '暂无创建时间'}></Cell>
            <Cell title="发货方式：" desc={detailInfo?.shipment_mode_name || '暂无发货方式'}></Cell>
            <Cell title="发货地址：" desc={!detailInfo.province_name ? '暂无发货地址' : `${detailInfo.province_name}${detailInfo.city_name}${detailInfo.district_name}${detailInfo.address_detail}`}></Cell>
          </View>
        </LayoutBlock>
        <LayoutBlock circle customStyle={{ padding: '24rpx' }}>
          <View className={styles.orderInfoTop}>附件</View>
          <Divider direction="horizontal" customClassName={styles.divider}></Divider>
          <UploadImage onlyRead={detailInfo?.status !== 0} onChange={handleUploadChange} defaultList={detailInfo?.delivery_appendix_url}></UploadImage>
        </LayoutBlock>
      </View>
      {detailInfo?.status === 0 && BottomBar()}
    </View>
  )
}
export default DeliveryDetail
