import type { ITouchEvent } from '@tarojs/components'
import { ScrollView, View } from '@tarojs/components'
import Taro from '@tarojs/taro'
import { memo, useCallback, useEffect, useMemo, useState } from 'react'
import styles from './index.module.scss'
import { formatRemoveHashTag } from '@/common/format'
import LabAndImg from '@/components/LabAndImg'
import NormalButton from '@/components/normalButton'
import Divider from '@/components/Divider'
import Tag from '@/components/tag'
import LayoutBlock from '@/components/layoutBlock'
import { goLink } from '@/common/common'

interface PropsType {
  data?: any
  cancel?: (e: ITouchEvent) => void
}
const ItemList = (props: PropsType) => {
  const { data, cancel } = props
  console.log('data===>', data)
  const labAndImgObj = useCallback((item) => {
    return { lab: item?.lab, rgb: item?.rgb, texture_url: item?.texture_url }
  }, [])
  // // 进入详情页
  const navTo = () => {
    goLink('/pages/getColorCard/colorCardDetail/index', { id: data?.order_id })
  }

  return (
    <LayoutBlock onClick={navTo} circle>
      <View className={styles.colorCardTopItem}>
        <View className={styles.colorCardOrderNo}>单号：{data.order_no}</View>
        <View className={styles.colorCardStatus}>{data.order_status_name}</View>
      </View>
      <View className={styles.colorCardFlexBox}>
        <View className={styles.colorCardPussName}>{data.purchaser_name}</View>
        {
          data.sale_user_name && <Tag type="primary" size="normal" circle customClassName={styles.colorCardTag} plain>
            {data.sale_user_name}
          </Tag>
        }
      </View>
      <Divider direction="horizontal" customStyles={{ margin: '12px 0' }}></Divider>
      <View className={styles.colorCardContBox}>
        <View style={{ minWidth: '24%' }}>
          <View className={styles.colorCardLeftCont}>
            <LabAndImg value={labAndImgObj(data.color_card_info?.[0])} />
          </View>
        </View>

        <View className={styles.colorCardRightCont}>
          <View className={styles.colorCardRightTop}>
            <View className={styles.colorCardProductName}>{data?.color_card_info[0]?.name}</View>
            <View className={styles.colorCardShipMode}>{data?.shipment_mode_name}</View>
          </View>
          <View className={styles.colorCardRightContTags}>
            {data?.color_card_info[0]?.affiliation_product?.map((item, index) => {
              return <Tag customStyle={{ marginRight: '5px', marginBottom: '2px', padding: '5px', background: '#e3ecff', color: '#558cff', borderColor: '#e3ecff' }} key={index} size="small" circle>{formatRemoveHashTag(item.code)}</Tag>
            })
            }

          </View>
          <View className={styles.colorCardColorsBox}>
            <View className={styles.colorCardColorName}>x{data?.color_card_info?.[0]?.count || 0}本</View>
          </View>
        </View>
      </View>
      <Divider direction="horizontal" customStyles={{ margin: '12px 0' }}></Divider>
      <View className={styles.colorCardBottomMsg}>
        <View className={styles.colorCardMsgLeft}>色卡信息：</View>
        <View className={styles.colorCardMsgRight}>{data?.color_card_count || 0} 种色卡，共 {data?.color_card_number || 0} 本</View>
      </View>
      <View className={styles.colorCardPaymentMethod}>
        <View className={styles.colorCardMsgRightOne}>快递到付</View>
      </View>
      <View className={styles.colorCardBottomBox}>
        {
          data?.order_status === 1 && <NormalButton
            customStyles={{ fontSize: '14px' }}
            type="info"
            size="normal"
            round
            plain
            onClick={cancel}
          >
      取消订单
          </NormalButton>
        }

      </View>
    </LayoutBlock>
  )
}
export default memo(ItemList)
