import { Image, Input, Text, View } from '@tarojs/components'
import Taro from '@tarojs/taro'
import type { FC } from 'react'
import { memo, useState } from 'react'
import { useDispatch } from 'react-redux'
import type { Dispatch } from 'redux'
import styles from './index.module.scss'
import Remind from './components/remind'
import Divider from '@/components/Divider'
import IconFont from '@/components/iconfont/iconfont'
import NormalButton from '@/components/normalButton'
import IconCard from '@/components/iconCard'
import { LoginApi } from '@/api'
import { alert, checkKey, goLink } from '@/common/common'
import useUserInfo from '@/use/useUserInfo'
import { BUSINESS_MANAGER_PHONE } from '@/common/constant'
import type { TabBarAction } from '@/reducers/tabBar'
import { INITIAL_STATE } from '@/reducers/tabBar'
import { TabBarType } from '@/constants/tabbar'
import Checkbox from '@/components/checkbox'

const QuickLogin: FC = () => {
  return (
    <View className={styles['quick-login']}>
      <Divider direction="horizontal" customClassName={styles['quick-login--divider']}>
        或通过以下方式登录
      </Divider>
      <View className={styles['quick-login--options']}>
        <IconCard iconName="icon-weixindenglu" title="微信登录" customClass={styles['quick-login--iconCard']}></IconCard>
      </View>
    </View>
  )
}
const QuickLoginWithMemo = memo(QuickLogin)
const Login: FC = () => {
  // const { selectedId, tabItem } = useSelector((state) => {
  //   return state.tabBarData
  // })
  const [isRemember, setRemember] = useState(false)

  const [eyesStatus, setEyesStatus] = useState(true)

  const handlePwdVisible = () => {
    setEyesStatus(value => !value)
  }

  const selectCallBack = () => {
    console.log('selectCallBack')
    setRemember(true)
  }
  const closeCallBack = () => {
    setRemember(false)
    console.log('closeCallBack')
  }

  const [account, setAccount] = useState('')
  const [password, setPassword] = useState('')

  const formatInput = (value: string) => {
    return value.trim()
  }
  const [selected, setSelected] = useState(false)
  const [showRemind, setShowRemind] = useState(false)
  const onSubmit = () => {
    setSelected(true)
    setShowRemind(false)
  }
  const handleInputAccount = (event: any) => {
    const formated = formatInput(event.detail.value)
    setAccount(formated)
  }
  const handleInputPassword = (event: any) => {
    const formated = formatInput(event.detail.value)
    setPassword(formated)
  }

  const { setToken, setUserInfo } = useUserInfo()

  const { fetchData } = LoginApi()
  const dispatch = useDispatch<Dispatch<TabBarAction>>()

  const handleLogin = async() => {
    if (account === '' || password === '') { return alert.error('账号密码不能为空！') }
    if (!selected) {
      setShowRemind(true)
      return
    }
    console.log('账户密码：', account, password)
    const res = await fetchData({
      account,
      password,
    })

    if (res.success) {
      alert.success('登陆成功')
      setToken(res.data.token)
      setUserInfo({ ...res.data })
      const tempTabBar = INITIAL_STATE.tabItem.filter(item => res.data.mp_role_access_list.includes(item.key))
      const dispatched = dispatch({ type: TabBarType.SET_TAB_ITEM, data: { tabItem: tempTabBar } })
      console.log('dispatch', dispatched)
      dispatch({ type: TabBarType.SET_SELECTED, data: { selectedId: tempTabBar?.[0].id } })
      Taro.switchTab({
        url: dispatched.data.tabItem?.[0]?.pagePath || '',
      })
    }
    else {
      alert.error('登陆失败')
    }
  }
  // 处理忘记密码的逻辑
  const handleForgetPwd = () => {
    Taro.showToast({ title: `联系客服:${BUSINESS_MANAGER_PHONE}`, icon: 'none' })
  }

  return (
    <View className={styles.login}>
      <Image className="full-100" mode="widthFix" src="https://cdn.zzfzyc.com/service_center/login_head.png"></Image>
      <View className={styles['login-panel']}>
        <View className={styles['login-title']}>
          <Text>您好，</Text>
          <Text>欢迎来到蜘蛛管家</Text>
        </View>
        <View className={styles['input-bar']}>
          <IconFont name="icon-yonghuming" size={48} color="#999999"></IconFont>
          <Divider direction="vertical" customStyles={{ height: '20px', marginLeft: '10px' }}></Divider>
          <Input onInput={handleInputAccount} className={styles['input-bar--account']} placeholder="请输入用户名"></Input>
        </View>
        <View className={styles['input-bar']}>
          <IconFont name="icon-mima" size={48} color="#999999"></IconFont>
          <Divider direction="vertical" customStyles={{ height: '20px', marginLeft: '10px' }}></Divider>
          <Input onInput={handleInputPassword} password={eyesStatus} className={styles['input-bar--password']} type="text" placeholder="请输入密码"></Input>
          <View onClick={handlePwdVisible}>
            {!eyesStatus
              ? (
                <IconFont name="icon-yanjing" size={48} color="#999999"></IconFont>
              )
              : (
                <IconFont name="icon-biyan" size={48} color="#999999"></IconFont>
              )}
          </View>
        </View>
        <View className={styles['login-options']}>
          <View className={styles['login-option']}>
            {/* <MCheckbox status={isRemember} size='small' circle onSelect={selectCallBack} onClose={closeCallBack}>
              记住密码
            </MCheckbox> */}

          </View>
          <View className={styles['login-option']} onClick={handleForgetPwd}>
            <Text className={styles.isForgetPwd}>忘记密码？</Text>
          </View>
        </View>
        <NormalButton circle onClick={handleLogin} customClassName={styles['login-button']} disabled={account === '' || password === ''}>
          登录
        </NormalButton>
        <View className={styles.regulations}>
          <Checkbox size="small" status={selected} onSelect={() => setSelected(true)} onClose={() => setSelected(false)} >
            <View className={styles.regulations_text}>
            我已阅读并同意
              <Text onClick={() => goLink('/pages/clauseDetail/index', { type: 'clause' })}>《服务条例》</Text>
            和
              <Text onClick={() => goLink('/pages/clauseDetail/index', { type: 'policy' })}>《隐私政策》</Text>
            </View>
          </Checkbox>
        </View>
        {/* H5 显示微信登陆 */}
        {process.env.TARO_ENV === 'h5' && <QuickLoginWithMemo />}
      </View>
      <Remind show={showRemind} onSubmit={onSubmit} onClose={() => setShowRemind(false)} />
    </View>
  )
}

export default Login
