import { Textarea, View } from '@tarojs/components'
import { useCallback, useEffect, useState } from 'react'
import styles from './index.module.scss'
import Popup from '@/components/popup'
import Tag from '@/components/tag'

interface Param {
  onBlur?: (val: any) => void
  onSave?: (val: string) => void
  defaultValue?: string
  recommend?: boolean
}
const recommendReport = [
  {
    id: 1,
    value: '拿色片',
  },
  {
    id: 2,
    value: '顺丰',
  },
  {
    id: 3,
    value: '客户自提',
  },
  {
    id: 4,
    value: '加急一下',
  },
  {
    id: 5,
    value: '物流到',
  },
  {
    id: 6,
    value: '中台秀银',
  },
  {
    id: 7,
    value: '中台晓茵',
  },
  {
    id: 8,
    value: '中台淑婷',
  },
  {
    id: 9,
    value: '中台结慧',
  },
  {
    id: 10,
    value: '辛苦了',
  },
]

const Remark = ({ onBlur, onSave, defaultValue = '', recommend = true }: Param) => {
  const [descData, setDescData] = useState({
    number: 0,
    value: '',
    count: 200,
  })

  const getDesc = (value) => {
    let res = value
    if (value.length > descData.count) {
      res = value.slice(0, descData.count)
    }
    setDescData({ ...descData, number: res.length, value: res })
  }

  const setSave = () => {
    onSave?.(descData.value)
  }
  useEffect(() => {
    getDesc(defaultValue)
  }, [defaultValue])

  const onClickTag = (item) => {
    setDescData(prev => ({
      ...prev,
      value: prev.value + item.value,
    }))
  }

  return (
    <View className={styles.order_popup}>
      <View className={styles.order_popup_title}>编辑备注</View>
      <View className={styles.order_popup_input}>
        <Textarea placeholder="请添加备注" value={descData?.value} maxlength={descData.count} cursorSpacing={100} onInput={e => getDesc(e.detail.value)} onBlur={e => onBlur?.(e)}></Textarea>
        <View className={styles.descDataNum}>{descData.number}/{descData.count}</View>
      </View>
      {
        recommend
          ? <View>
            <View className={styles.recommend_remark}>推荐备注</View>
            <View className={styles.recommend_remark_content}>
              {
                recommendReport.map((item) => {
                  return <Tag type="info" plain onClick={() => onClickTag(item)} circle customClassName={styles.tag} key={item.id}>{item.value}</Tag>
                })
              }
            </View>
          </View>
          : null
      }

      <View className={styles.order_save_address} onClick={() => setSave()}>保存</View>
    </View>
  )
}

export default Remark
