import { View } from '@tarojs/components'
import { FC, forwardRef, memo, useEffect, useImperativeHandle, useMemo, useRef, useState } from 'react'
import styles from './index.module.scss'
import { EnumMarketingDepartmentApi } from '@/api/index'
import DropDownItem from '@/components/dropDown-item'
import FilterButton from '@/components/filterButton'

type ChangedValue = string | number

interface SelectSaleTypeProps {
  onChange?: (value: ChangedValue) => void
  onCloseOverlay?: () => void
}

interface EnumList {
  id: number
  code: string
  name: string
}
const manual = [
  { id: 1, code: '0', name: '进行中' },
  { id: 2, code: '1', name: '有异常' },
  { id: 3, code: '2', name: '已完成' },
  { id: 4, code: '2', name: '已取消' },
  { id: 5, code: '2', name: '超时完成' },
]
// 处理状态
const SelectProcessingStatus = (props: SelectSaleTypeProps, ref) => {
  const selectName = '处理状态'

  const { onChange, onCloseOverlay } = props
  console.log(props)
  const { fetchData } = EnumMarketingDepartmentApi()

  const [enumList, setEnumList] = useState<EnumList[]>([])

  const getData = async() => {
    const res = await fetchData()
    setEnumList([{ id: -1, code: '', name: '全部' }, ...manual])
  }

  useEffect(() => {
    getData()
  }, [])

  const [currentValue, setCurrentValue] = useState<ChangedValue>(-1)

  const handleClick = (value: ChangedValue) => {
    setCurrentValue(value)
    onChange?.(value)
  }
  const dropDownRef = useRef<any>(null)

  useImperativeHandle(
    ref,
    () => {
      return {
        show: dropDownRef.current.show,
        showPopup: dropDownRef.current.showPopup,
        closePopup: dropDownRef.current.closePopup,
      }
    },
    [dropDownRef.current],
  )

  const displayTitle = useMemo(() => {
    if (currentValue === -1) {
      return selectName
    }
    return enumList.length ? enumList.filter(option => option.id === currentValue)?.[0]?.name : selectName
  }, [enumList, currentValue])

  return (
    <DropDownItem title={displayTitle} value={currentValue} activeColor="#337fff" ref={dropDownRef} onCloseOverlay={onCloseOverlay}>
      <View className={styles.grid}>
        {!!enumList.length
          && enumList.map((item: EnumList, key) => {
            return (
              <FilterButton circle key={key} isActive={item.id === currentValue} onClick={() => handleClick(item.id)}>
                {item.name}
              </FilterButton>
            )
          })}
      </View>
    </DropDownItem>
  )
}
export default memo(forwardRef(SelectProcessingStatus))
