import { Text, View } from '@tarojs/components'
import classNames from 'classnames'
import React from 'react'
import styles from './index.module.scss'

export enum IStatusType {
  PENDING = 'pending',
  WAITING = 'waiting',
  SUCCESS = 'success',
  FAIL = 'fail',
  CANCAL = 'cancal',
}

interface statusList {
  id: number
  colorClassName?: IStatusType
  type?: IStatusType
  typeName?: string
  children?: statusList[]
  showText?: boolean
}

interface StatusLineProps {
  showStatusText?: boolean
  line: statusList[]
}

const StatusLine = (props: StatusLineProps) => {
  const { line } = props
  return <View className={styles.process}>
    {
      line.map((item) => {
        if (item.children && item.children.length > 0) {
          return <View className={classNames(styles.block, styles.col)} key={item.id}>
            {
              item.children.map((subItem, index) => {
                return <View className={classNames(styles.block, subItem?.colorClassName && styles[subItem.colorClassName], styles.subBlock)} key={subItem.id}>
                  {
                    subItem.showText && <Text className={classNames(styles.text, styles[`text${index + 1}`])}>{subItem.typeName}</Text>
                  }
                </View>
              })
            }
          </View>
        }
        else {
          return <View className={classNames(styles.block, item?.colorClassName && styles[item.colorClassName])} key={item.id}>
            {
              item.showText && <Text className={styles.text}>{item.typeName}</Text>
            }
          </View>
        }
      })
    }
  </View>
}
export default StatusLine
