import { Text, View } from '@tarojs/components'
import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react'
import dayjs from 'dayjs'
import classnames from 'classnames'
import styles from './index.module.scss'
import OrderItem from './components/orderItem'
import SelectProcessingStatus from './components/SelectProcessingStatus'
import StatusLine, { IStatusType } from './components/statusLine'
import SelectGroup from '@/components/SelectGroup'
import InfiniteScroll from '@/components/infiniteScroll'
import type { ChangedValue } from '@/components/SelectTimePicker'
import SelectTimePicker from '@/components/SelectTimePicker'
import SelectSaleType from '@/components/SelectSaleType'
import { dataLoadingStatus, getFilterData } from '@/common/util'
import { ClientListApi } from '@/api/order'
import IconText from '@/components/iconText'
import Dialog from '@/components/Dialog'
import NormalButton from '@/components/normalButton'
import { GetOrderEarlyWarningListApi } from '@/api/orderWarning'
import SelectOrderType from '@/components/SelectOrderType'
import { alert } from '@/common/common'

const FilterTimeOptions = {
  1: {
    name: '今天',
    date_min: `${dayjs(new Date())
      .add(-1, 'day')
      .format('YYYY-MM-DD')} 19:00:00`,
    date_max: `${dayjs(new Date())
      .format('YYYY-MM-DD')} 19:00:00`,
  },
  2: {
    name: '昨日',
    date_min: `${dayjs(new Date())
      .add(-2, 'day')
      .format('YYYY-MM-DD')} 19:00:00`,
    date_max: `${dayjs(new Date())
      .add(-1, 'day')
      .format('YYYY-MM-DD')} 19:00:00`,
  },
  // 2: {
  //   name: '近7日',
  //   date_min: `${dayjs(new Date())
  //     .add(-7, 'day')
  //     .format('YYYY-MM-DD')} 00:00:00`,
  //   date_max: `${dayjs(new Date())
  //     .add(0, 'day')
  //     .format('YYYY-MM-DD')} 00:00:00`,
  // },
  // 3: {
  //   name: '近30日',
  //   date_min: `${dayjs(new Date())
  //     .add(-30, 'day')
  //     .format('YYYY-MM-DD')} 00:00:00`,
  //   date_max: `${dayjs(new Date())
  //     .add(0, 'day')
  //     .format('YYYY-MM-DD')} 00:00:00`,
  // },
  // 4: {
  //   name: '近90日',
  //   date_min: `${dayjs(new Date())
  //     .add(-90, 'day')
  //     .format('YYYY-MM-DD')} 00:00:00`,
  //   date_max: `${dayjs(new Date())
  //     .add(0, 'day')
  //     .format('YYYY-MM-DD')} 00:00:00`,
  // },
}

const statusLine = [
  {
    id: 1,
    colorClassName: IStatusType.SUCCESS,
    type: IStatusType.SUCCESS,
    typeName: '开始',
    showText: true,
  },
  {
    id: 2,
    colorClassName: IStatusType.SUCCESS,
    type: IStatusType.SUCCESS,
    typeName: '接单',
    showText: true,
  },
  {
    id: 3,
    colorClassName: IStatusType.SUCCESS,
    type: IStatusType.SUCCESS,
    typeName: '配布',
    showText: true,
  },
  {
    id: 4,
    children: [
      {
        id: 5,
        colorClassName: IStatusType.SUCCESS,
        type: IStatusType.SUCCESS,
        typeName: '提货',
        showText: true,
      },
      {
        id: 6,
        colorClassName: IStatusType.SUCCESS,
        type: IStatusType.SUCCESS,
        typeName: '收款',
        showText: true,
      },
    ],
  },

  {
    id: 7,
    colorClassName: IStatusType.SUCCESS,
    type: IStatusType.SUCCESS,
    typeName: '出仓',
    showText: true,
  },
  {
    id: 8,
    colorClassName: IStatusType.SUCCESS,
    type: IStatusType.SUCCESS,
    typeName: '收货',
    showText: true,
  },
]

interface ISearchField {
  order_start_time: string
  order_end_time: number | string
  sale_mode: string
  handle_status: string
}

// 订单预警分析
const OrderWarningAnalysis = () => {
  const { fetchData, state } = GetOrderEarlyWarningListApi()

  const [refresherTriggeredStatus, setRefresherTriggeredStatus] = useState(false)
  const getData = async() => {
    const res = await fetchData(getFilterData(searchField))
    setOrderData(e => ({ ...e, list: res.data?.list, total: res.data?.total }))
    setRefresherTriggeredStatus(false)
  }

  const [searchField, setSearchField] = useState<ISearchField>({
    order_start_time: FilterTimeOptions[1].date_min,
    order_end_time: FilterTimeOptions[1].date_max,
    sale_mode: '',
    handle_status: '',
  })

  const onChangeOrderType = (value) => {
    setSearchField((e: ISearchField) => ({
      ...e,
      sale_mode: value === -1 ? '' : value,
    }))
  }
  const limit = 1
  const onChangeTimePicker = useCallback((value: ChangedValue) => {
    console.log('value', value)
    let [date_min, date_max] = value
    date_min = dayjs(date_min).subtract(1).format('YYYY-MM-DD 19:00:00')
    date_max = dayjs(date_max).format('YYYY-MM-DD 19:00:00')
    const diff = dayjs(date_max).diff(dayjs(date_min), 'days')
    console.log('diff', diff)
    if (diff > limit) {
      return alert.none('日期范围不能超过1天')
    }
    setSearchField((e: ISearchField) => ({
      ...e,
      order_start_time: date_min,
      order_end_time: date_max,
    }))
  }, [])
  const onChangeStatus = (value) => {
    setSearchField((e: ISearchField) => ({
      ...e,
      handle_status: value === -1 ? '' : value,
    }))
  }

  const [orderData, setOrderData] = useState<{ list: any[]; total: number }>({ list: [], total: 0 })
  // 数据加载状态
  const statusMore = useMemo(() => {
    return dataLoadingStatus({ list: orderData.list, total: orderData.list.length, status: state.loading! })
  }, [orderData, state])

  // const pageNum = useRef({ size: searchField.size, page: searchField.page })
  // const getScrollToLower = useCallback(() => {
  //   if (orderData.list.length < orderData.total) {
  //     pageNum.current.page++
  //     const size = pageNum.current.size * pageNum.current.page
  //     setSearchField({ ...searchField, size })
  //   }
  // }, [orderData])
  const getRefresherRefresh = async() => {
    // pageNum.current.page = 1
    setRefresherTriggeredStatus(true)
    setSearchField(val => ({ ...val }))
  }

  // 监听筛选条件变化
  useEffect(() => {
    getData()
  }, [searchField])

  const [showDialog, setShowDialog] = useState(false)

  const handleShowTips = (bool) => {
    setShowDialog(bool)
  }

  const handleConfirm = () => {
    setShowDialog(false)
  }

  const selectTimePickerRef = useRef<any>(null)
  const selectProcessingStatusRef = useRef<any>(null)
  const selectOrderTypeRef = useRef<any>(null)
  return <View className={styles.main}>
    <View className={styles.topBox}>
      <View className={styles.menuBox} >
        <SelectGroup>
          <SelectOrderType ref={selectOrderTypeRef} onChange={onChangeOrderType} onCloseOverlay={() => true}></SelectOrderType>
          <SelectProcessingStatus ref={selectProcessingStatusRef} onChange={onChangeStatus} onCloseOverlay={() => true}></SelectProcessingStatus>
          <SelectTimePicker ref={selectTimePickerRef} defaultValue={1} onChange={onChangeTimePicker} onCloseOverlay={() => true} timeOptions={FilterTimeOptions}></SelectTimePicker>
        </SelectGroup>
      </View>
    </View>
    <View className={styles.totalFont}>
      <View onClick={handleShowTips}>
        <IconText direction="right" iconName="icon-tishi1" iconSize={30} color="#aeaeae" svg text="预警说明"></IconText>
      </View>
      <Text>共 {orderData?.total || 0} 单</Text>
    </View>
    <View className={styles.order_list}>
      <InfiniteScroll
        statusMore={statusMore}
        // selfonScrollToLower={getScrollToLower}
        refresherEnabled
        refresherTriggered={refresherTriggeredStatus}
        selfOnRefresherRefresh={getRefresherRefresh}
      >
        {orderData?.list?.map((item, index) => {
          return (
            <OrderItem key={index} data={item}></OrderItem>
          )
        })}
      </InfiniteScroll>
    </View>
    <Dialog show={showDialog} onChange={handleShowTips}>
      <View className={classnames(styles.statusDialog)}>
        <View className={styles.title}>进度条信息</View>
        <View className={styles.statusLine}>
          <StatusLine line={statusLine} />
        </View>
        <View className={styles.explanation}>
          <View className={styles.explanation_block}>
            <View className={classnames(styles.explanation_status, styles.explanation_waiting)}></View>
            <View className={styles.explanation_text}>待进行</View>
          </View>
          <View className={styles.explanation_block}>
            <View className={classnames(styles.explanation_status, styles.explanation_success)}></View>
            <View className={styles.explanation_text}>正常进行</View>
          </View>
          <View className={styles.explanation_block}>
            <View className={classnames(styles.explanation_status, styles.explanation_fail)}></View>
            <View className={styles.explanation_text}>超时完成</View>
          </View>
          <View className={styles.explanation_block}>
            <View className={classnames(styles.explanation_status, styles.explanation_pending)}></View>
            <View className={styles.explanation_text}>进行中异常</View>
          </View>
          <View className={styles.explanation_block}>
            <View className={classnames(styles.explanation_status, styles.explanation_cancel)}></View>
            <View className={styles.explanation_text}>已取消</View>
          </View>
        </View>
        <NormalButton circle type="primary" customClassName={styles.btn} onClick={handleConfirm}>确认</NormalButton>
      </View>
    </Dialog>
  </View>
}
export default OrderWarningAnalysis
