import { Button, Image, Text, Textarea, View } from '@tarojs/components'
import Taro, { useDidShow, usePullDownRefresh, useRouter } from '@tarojs/taro'
import type { ReactNode } from 'react'
import { memo, useCallback, useEffect, useMemo, useRef, useState } from 'react'
import styles from './index.module.scss'
import OrderState from './components/orderState'
import AddressDetailBox from './components/addressDetailBox'
import { formatDateTime, formatHashTag, formatImgUrl, formatPriceDiv, formatWeightDiv } from '@/common/format'
import {
  AuditReturnApplyOrder,
  GetWarehouseList,
  MpReturnApplyOrder,
  RefuseReturnApplyOrder,
  ReturnApplyOrderCancel,
  ReturnApplyOrderUploadAccessory,
} from '@/api/refound'
import { IMG_CND_Prefix } from '@/common/constant'
import Popup from '@/components/popup'
import UploadImage from '@/components/uploadImage'
import { alert, checkKey } from '@/common/common'
import RefundCodelist from '@/components/RefundCodelist'
import LabAndImg from '@/components/LabAndImg'
import NormalButton from '@/components/normalButton'
import Divider from '@/components/Divider'
import Cell from '@/components/cell'

enum ReturnEnum {
  Refunded = 3, // 已退款
  RefundedBefore = 5, // 待退款-确认单
  RefundedAfter = 6, // 待退款
}

enum ReturnType {
  TypeOne = 2, // 预收退款
}

interface Obs {
  title?: string
  modeName?: string
  showMode?: boolean
  children?: ReactNode
  clickNode?: () => void
}

const DefaultBox = (props: Obs) => {
  const {
    title = '标题',
    modeName = '大货',
    showMode = false,
    children,
    clickNode,
  } = props

  return (
    <View className={styles.defaltBox}>
      <View className={styles.titleBox}>
        <View className={styles.title}>{title}</View>
        {
          showMode && <View className={styles.modeName} onClick={() => clickNode?.()}>{modeName}</View>
        }
      </View>
      <View className={styles.modeLine}></View>
      {children}
    </View>
  )
}
const DefaultBoxWithMemo = memo(DefaultBox)

interface PropGoods {
  // item?: {
  //     code?: string | number
  // }
  list: any[]
  obj: any
}

const GoodsItem = (porps: PropGoods) => {
  const { list = [], obj = {} } = porps

  const labAndImgObj = useCallback((item) => {
    return { lab: item.lab, rgb: item.rgb, texture_url: item.texture_url }
  }, [])
  return (
    <>
      {
        list.map((item, index) => {
          return (
            <View className={styles.goodsBox} key={index}>
              <View className={styles.goodsProduct}>{formatHashTag(item.code, item.name)}</View>
              {
                item.product_colors.map((it, inx) => {
                  return (
                    <View className={styles.itemGoods} key={inx}>
                      {
                        it.texture_url === ''
                                                && <Image className={styles.itemPic} mode="aspectFill" src="https://test.cdn.zzfzyc.com/mall/no_img.png"></Image>
                      }
                      {
                        it.texture_url !== ''
                                                && <View className={styles.itemPic}>
                                                  <LabAndImg value={labAndImgObj(it)} />
                                                </View>
                      }
                      <View className={styles.clear}> </View>
                      <View className={styles.itemRight}>
                        <View className={styles.item_right_top}>
                          <View className={styles.itemName}>{formatHashTag(it?.code, it?.name)}</View>
                          {/* <View className={styles.itemNums}>x{obj?.sale_mode === 0 ? it.roll : it.length / 100}{obj?.sale_mode === 0 ? '条' : 'm'}</View> */}
                          {
                            obj.sale_mode == 0 && <View className={styles.itemNums}>x{obj.stage == 0 || obj.stage == 1 || obj.stage == 2
                              ? it.roll
                              : obj.stage == 5 || obj.stage == 6 || obj.stage == 3
                                ? it.return_roll
                                : it.roll}{obj?.sale_mode == 0 ? '条' : 'm'}</View>
                          }
                          {
                            obj.sale_mode != 0 && <View className={styles.itemNums}>x{it.length / 100}m</View>
                          }
                        </View>
                        <View className={styles.item_right_Bottom}>
                          <View className={styles.itemMoney}>¥{it.sale_price / 100}/{obj?.sale_mode == 0 ? 'kg' : 'm'}</View>
                          {/* <View className={styles.itemMoneyOne}>¥{formatPriceDiv(it.total_sale_price)}</View> */}
                        </View>
                      </View>
                    </View>
                  )
                })
              }
            </View>
          )
        })
      }
    </>

  )
}
const GoodsItemWithMemo = memo(GoodsItem)

const RefundDetail = () => {
  // 页面下拉刷新
  const router = useRouter()
  // 查看图片
  const [ShowPic, setShowPic] = useState(false)

  const [PicList, setPicList] = useState<any[]>([])
  // 订单信息文字数组
  const [orderMsg, setOrderMsg] = useState<any[]>([
    {
      leftTitle: '售后编号：',
      rightTitle: '------',
      showBtn: true,
    },
    {
      leftTitle: '订单号：',
      rightTitle: '------',
      showBtn: true,
    },
    {
      leftTitle: '退货原因：',
      rightTitle: '------',
    },
    {
      leftTitle: '退货说明：',
      rightTitle: '------',
    },
    {
      leftTitle: '其他说明：',
      rightTitle: '------',
    },
    {
      leftTitle: '货物状况：',
      rightTitle: '------',
    },
    {
      leftTitle: '申请时间：',
      rightTitle: '------',
    },
  ])
  // 获取详情
  const { fetchData: detailFetch } = MpReturnApplyOrder()
  const [DeatailObj, setDeatailObj] = useState<any>({})
  const getDetail = async() => {
    Taro.showLoading({
      title: '加载中...',
      mask: true,
    })
    const res = await detailFetch({ id: Number(router.params.id) })
    if (res.data) {
      setDeatailObj(res.data)
      orderMsg.forEach((it) => {
        if (it.leftTitle === '售后编号：') {
          it.rightTitle = res.data.return_order_no
        }
        if (it.leftTitle === '订单号：') {
          it.rightTitle = res.data.order_no
        }
        if (it.leftTitle === '退货原因：') {
          it.rightTitle = res.data.return_reason_name
        }
        if (it.leftTitle === '退货说明：') {
          it.rightTitle = res.data.return_explain_name
        }
        if (it.leftTitle === '其他说明：') {
          it.rightTitle = res.data.reason_describe
        }
        if (it.leftTitle === '货物状况：') {
          it.rightTitle = res.data.goods_status_name
        }
        if (it.leftTitle === '申请时间：') {
          it.rightTitle = formatDateTime(res.data.apply_time)
        }
      })
      setOrderMsg([...orderMsg])
      setPicList([...res.data?.accessory_url !== null ? res.data?.accessory_url : []])
      Taro.hideLoading()
      // 小程序提供的api，通知页面停止下拉刷新效果
      Taro.stopPullDownRefresh()
    }
  }

  // 分辨用质检中还是质检前的数组
  const productList = useMemo(() => {
    if (DeatailObj.stage === 0
            || (DeatailObj.sale_mode == 1 && DeatailObj.type == 1) || DeatailObj.stage === 1 || DeatailObj.stage === 2) {
      return DeatailObj.product_list
    }
    else {
      return DeatailObj.quality_check_pass_product
    }
  }, [DeatailObj])

  // 复制功能
  const clipboardData = (val) => {
    Taro.setClipboardData({
      data: val || '',
      success(res) {
        Taro.showToast({
          icon: 'none',
          title: '复制成功',
        })
      },
    })
  }

  // 取消退货
  const { fetchData: cancelFetch } = ReturnApplyOrderCancel()
  const handCancle = async() => {
    Taro.showModal({
      content: '确定要取消吗？',
      confirmText: '确认',
      cancelText: '取消',
      async success(res) {
        if (res.confirm) {
          const res = await cancelFetch({ id: Number(DeatailObj.id) })
          Taro.showLoading({
            title: '请稍等...',
            mask: true,
          })
          if (res.msg === 'success') {
            Taro.showToast({
              title: '取消成功',
            })
            Taro.hideLoading()
            getDetail()
          }
          else {
            Taro.showToast({
              title: res.msg,
              icon: 'error',
            })
          }
        }
      },
    })
  }

  const handUp = () => {
    setShowPic(true)
  }

  const onlyRead = useMemo(() => {
    if (DeatailObj.stage == 1) {
      return false
    }
    else {
      return true
    }
  }, [DeatailObj])

  // 获取图片列表
  const picUrl = useRef([])
  const getImageList = useCallback((list) => {
    picUrl.current = list
  }, [])

  // 提交附件
  const { fetchData: sureFetch } = ReturnApplyOrderUploadAccessory()
  const handSure = () => {
    let arr: any = []
    arr = [...picUrl.current]
    if (arr == 0) {
      return alert.error('请先添加附件')
    }
    Taro.showModal({
      content: '确定要提交吗？',
      confirmText: '确认',
      cancelText: '取消',
      async success(res) {
        if (res.confirm) {
          const res = await sureFetch({ id: Number(DeatailObj.id), accessory_url: arr })
          Taro.showLoading({
            title: '请稍等...',
            mask: true,
          })
          if (res.msg === 'success') {
            Taro.showToast({
              title: '成功',
            })
            setShowPic(false)
            Taro.hideLoading()
            getDetail()
          }
          else {
            Taro.showToast({
              title: res.msg,
              icon: 'error',
            })
          }
        }
      },
    })
  }
  const proview = (item) => {
    Taro.previewImage({
      current: IMG_CND_Prefix + item, // 当前显示图片的 http 链接
      urls: DeatailObj.fabric_piece_accessory_url.map((item) => {
        item = IMG_CND_Prefix + item
        return item
      }), // 需要预览的图片 http 链接列表
    })
  }
  const [ShowCode, setShowCode] = useState(false)
  const handSee = () => {
    setShowCode(true)
  }
  usePullDownRefresh(() => {
    getDetail()
  })
  useEffect(() => {
    getDetail()
    getWarehouseList()
  }, [])

  // 待退款和已退款需要变更字段
  const totalGoods = useMemo(() => {
    if ((DeatailObj.stage == ReturnEnum.Refunded || DeatailObj.stage == ReturnEnum.RefundedBefore || DeatailObj.stage == ReturnEnum.RefundedAfter) && DeatailObj.type != ReturnType.TypeOne) {
      return DeatailObj.quality_check_pass_fabrics
    }
    if ((DeatailObj.stage != ReturnEnum.Refunded || DeatailObj.stage != ReturnEnum.RefundedBefore || DeatailObj.stage != ReturnEnum.RefundedAfter) && DeatailObj.type != ReturnType.TypeOne) {
      return DeatailObj.total_fabrics
    }
    if (DeatailObj.type == ReturnType.TypeOne) {
      return DeatailObj.total_fabrics
    }
  }, [DeatailObj])
  // 待退款和已退款需要变更字段
  const totalColors = useMemo(() => {
    if ((DeatailObj.stage == ReturnEnum.Refunded || DeatailObj.stage == ReturnEnum.RefundedBefore || DeatailObj.stage == ReturnEnum.RefundedAfter) && DeatailObj.type != ReturnType.TypeOne) {
      return DeatailObj.quality_check_pass_colors
    }
    if ((DeatailObj.stage != ReturnEnum.Refunded || DeatailObj.stage != ReturnEnum.RefundedBefore || DeatailObj.stage != ReturnEnum.RefundedAfter) && DeatailObj.type != ReturnType.TypeOne) {
      return DeatailObj.total_colors
    }
    if (DeatailObj.type == ReturnType.TypeOne) {
      return DeatailObj.total_colors
    }
  }, [DeatailObj])
  // 待退款和已退款需要变更字段
  const totalNums = useMemo(() => {
    if ((DeatailObj.stage == ReturnEnum.Refunded || DeatailObj.stage == ReturnEnum.RefundedBefore || DeatailObj.stage == ReturnEnum.RefundedAfter) && DeatailObj.sale_mode == 0 && DeatailObj.type != ReturnType.TypeOne) {
      return DeatailObj.quality_check_pass_number
    }
    if ((DeatailObj.stage == ReturnEnum.Refunded || DeatailObj.stage == ReturnEnum.RefundedBefore || DeatailObj.stage == ReturnEnum.RefundedAfter) && DeatailObj.sale_mode != 0 && DeatailObj.type != ReturnType.TypeOne) {
      return DeatailObj.quality_check_pass_number / 100
    }
    if ((DeatailObj.stage != ReturnEnum.Refunded || DeatailObj.stage != ReturnEnum.RefundedBefore || DeatailObj.stage != ReturnEnum.RefundedAfter) && DeatailObj.sale_mode == 0 && DeatailObj.type != ReturnType.TypeOne) {
      return DeatailObj.return_roll
    }
    if ((DeatailObj.stage != ReturnEnum.Refunded || DeatailObj.stage != ReturnEnum.RefundedBefore || DeatailObj.stage != ReturnEnum.RefundedAfter) && DeatailObj.sale_mode != 0 && DeatailObj.type != ReturnType.TypeOne) {
      return DeatailObj.total_number / 100
    }
    if (DeatailObj.type == ReturnType.TypeOne) {
      return DeatailObj.total_number / 100
    }
  }, [DeatailObj])

  const { fetchData: auditApi } = AuditReturnApplyOrder()
  const { fetchData: refuseApi } = RefuseReturnApplyOrder()

  const [showSaleAfterPopup, setShowSaleAfterPopup] = useState(false)

  const handleShowPopup = () => {
    setShowSaleAfterPopup(true)
  }

  const navTo = () => {
    Taro.navigateTo({
      url: '/pages/wareHousePage/index',
    })
  }

  const [remarkValue, setRemarkValue] = useState('')
  const getDesc = (e) => {
    setRemarkValue(e.detail.value)
  }
  const onBlur = (e) => {
    setRemarkValue(e.detail.value)
  }

  const handleRefuseSubmit = async() => {
    if (!remarkValue) {
      return alert.error('请输入拒绝原因')
    }
    const res = await refuseApi({
      remark: remarkValue,
      return_apply_order_id: Number(router.params.id),
    })
    if (res.success) {
      getDetail()
      alert.success('审核成功')
      setShowSaleAfterPopup(false)
    }
    else {
      alert.error(res.msg)
    }
  }
  const handleAuditSubmit = async() => {
    if (clientObj.clientId === -1 || !clientObj.clientId) {
      alert.error('请选择仓库')
    }
    const res = await auditApi({
      remark: remarkValue,
      return_apply_order_id: Number(router.params.id),
      warehouse_id: clientObj.clientId,
    })
    if (res.success) {
      getDetail()
      alert.success('审核成功')
      setShowSaleAfterPopup(false)
    }
    else {
      alert.error(res.msg)
    }
  }

  const [clientObj, setclientObj] = useState({
    clientId: -1,
    clientName: '',
  })
  const { fetchData } = GetWarehouseList()
  const [clienList, setclienList] = useState<any[]>([])

  const getWarehouseList = async() => {
    const res = await fetchData()
    setclientObj({
      clientId: res.data.list?.length > 0 ? res.data.list[0]?.id : -1,
      clientName: res.data.list?.length > 0 ? res.data.list[0]?.name : '',
    })
    setclienList([...res.data.list])
  }

  useDidShow(() => {
    // 获取选择的客户
    const pages = Taro.getCurrentPages()
    const currPage = pages[pages.length - 1] // 获取当前页面\
    console.log('currPage', currPage)
    // 判断是否有跳转选择客户
    if (currPage.data?.clientId) {
      setclientObj({
        clientId: currPage.data?.clientId,
        clientName: currPage.data?.clientName,
      })
    }
    // 默认仓库
    if (!currPage.data?.clientId) {
      console.log('默认仓库')
      setclientObj({
        clientId: clienList.length > 0 ? clienList[0]?.id : -1,
        clientName: clienList.length > 0 ? clienList[0]?.name : '',
      })
    }
  })

  return (
    <View className={styles.main}>
      <View className={styles.context}>
        <OrderState orderInfo={DeatailObj} />
        <AddressDetailBox
          obj={DeatailObj}
          handUp={handUp}
        ></AddressDetailBox>
        <DefaultBoxWithMemo
          showMode
          title="客户信息"
          modeName={DeatailObj.sale_mode_name}
        >
          <View className={styles.pussBox}>
            <View className={styles.pussName}>{DeatailObj.purchaser_name}</View>
            <View className={styles.pussPhone}>{DeatailObj.purchaser_phone}</View>
          </View>
        </DefaultBoxWithMemo>
        <DefaultBoxWithMemo title="退款信息">
          <GoodsItemWithMemo list={productList} obj={DeatailObj}></GoodsItemWithMemo>
          <View className={styles.totalBox}>
            <View className={styles.totalLeft}>总计</View>
            <View className={styles.totalRight}>{totalGoods} 种面料，{totalColors} 种颜色，共 { totalNums} {DeatailObj?.sale_mode == 0 ? '条' : '米'}</View>
          </View>
          {
            (DeatailObj.stage == 3)
                        && <View className={styles.totalBox}>
                          <View className={styles.totalLeft}>退款金额</View>
                          <View className={styles.totalRight} style={{ color: '#F64861' }}>￥{
                            formatPriceDiv(DeatailObj.refund_amount)
                          }</View>
                        </View>

          }
        </DefaultBoxWithMemo>
        <DefaultBoxWithMemo title="订单信息">
          {
            orderMsg.map((item, index) => {
              return (
                <View className={styles.detailBox} key={index}>
                  <View className={styles.detailFont}>
                    {item.leftTitle}
                  </View>
                  <View className={styles.detailRightFlex}>
                    <View className={styles.detailRight}>{item.rightTitle}</View>
                    {item.showBtn && <View className={styles.detailBtn} onClick={() => clipboardData(item.rightTitle)}>复制</View>}
                  </View>

                </View>
              )
            })
          }

        </DefaultBoxWithMemo>
        <DefaultBoxWithMemo title="售后图片">
          <View className={styles.picBox}>
            {
              DeatailObj?.fabric_piece_accessory_url?.length > 0 && DeatailObj?.fabric_piece_accessory_url.map((item, index) => {
                return (
                  <Image className={styles.pic} lazyLoad key={index} onClick={() => proview(item)}
                    mode="aspectFill" src={IMG_CND_Prefix + item}
                  ></Image>
                )
              })
            }
            {
              DeatailObj?.fabric_piece_accessory_url?.length === 0 && <View className={styles.noPic}>暂无图片</View>
            }
          </View>
        </DefaultBoxWithMemo>
      </View>
      <View className={styles.bottomMuck}>
        {
          checkKey('return_and_refund_audit')
            ? <>
              {
                DeatailObj.stage === 0 && <NormalButton type="primary" plain round onClick={handleShowPopup}>售后审批</NormalButton>
              }
            </>
            : null
        }
        {
          (DeatailObj.stage === 0 || DeatailObj.stage === 1) && <NormalButton type="info" plain round customClassName={styles.cancleBtn} onClick={() => handCancle()}>取消退货</NormalButton>
        }
        {
          (DeatailObj.sale_mode == 0 && DeatailObj.stage == 3 && (DeatailObj.type == 1 || DeatailObj.type == 3)) && <NormalButton type="info" plain round customClassName={styles.seeButon} onClick={() => { handSee() }}>查看码单</NormalButton>
        }

      </View>
      <RefundCodelist
        obj={DeatailObj}
        // picUrl={PicUrl}
        showPopup={ShowCode}
        popupClose={() => setShowCode(false)}
      ></RefundCodelist>
      <Popup title="售后审批" show={showSaleAfterPopup} onClose={() => setShowSaleAfterPopup(false)}>
        <View className={styles.popup}>
          <Cell title={
            <View className={styles.title}>退回仓库<Text style={{ color: 'red' }}>*</Text></View>
          } desc={
            clientObj.clientId !== -1 ? clientObj.clientName : '请选择仓库'
          } isLink onClick={navTo}
          ></Cell>
          <Divider direction="horizontal" customStyles={{ margin: '0' }}></Divider>
          <View className={styles.title}>售后备注</View>
          <View style={{ marginTop: '24rpx' }}>
            <Textarea className={styles.textarea} placeholder="请输入售后备注信息" value={remarkValue} cursorSpacing={100} onInput={getDesc} onBlur={onBlur}></Textarea>
          </View>
          <View className={styles.popupBtn}>
            <NormalButton type="danger" customClassName={styles.button} plain round onClick={handleRefuseSubmit} >拒绝</NormalButton>
            <NormalButton type="primary" customClassName={styles.button} round onClick={handleAuditSubmit} >同意</NormalButton>
          </View>
        </View>
      </Popup>
      <Popup title="查看/修改物流回单" show={ShowPic} onClose={() => setShowPic(false)}>
        <>
          <UploadImage onChange={getImageList} defaultList={PicList} onlyRead={onlyRead} />
          {

            !onlyRead && <View className={styles.picBtn} onClick={() => handSure()}>确认修改</View>
          }
        </>
      </Popup>
    </View>
  )
}
export default RefundDetail
