import { View } from '@tarojs/components'
import Taro, { useDidShow } from '@tarojs/taro'
import React, { memo, useCallback, useEffect, useMemo, useRef, useState } from 'react'
import classnames from 'classnames'
import styles from './index.module.scss'
import Goods from './components/goods'
import Search from '@/components/search'
import Empty from '@/components/empty'
import { MpProductList } from '@/api/search'
import { debounce, getFilterData } from '@/common/util'
import ShopCart from '@/components/shoppingCart'
import {
  ClientListApi,
  DeleteSearchHistoryList,
  MpSearchHistory,
  MpSearchHistoryList,
} from '@/api/order'
import IconFont from '@/components/iconfont/iconfont'
import { alert } from '@/common/common'

const SearchPage = () => {
  const [clientObj, setclientObj] = useState({
    clientId: -1,
    clientName: '',
  })

  // 获取客户
  const [clienList, setclienList] = useState<any[]>([])
  const { fetchData: listFetchData } = ClientListApi()
  const getClient = async() => {
    const res = await listFetchData({
      page: 1, size: 10,
    })
    setclientObj({
      clientId: res.data.list.length > 0 ? res.data.list[0]?.id : -1,
      clientName: res.data.list.length > 0 ? res.data.list[0]?.name : '',
    })
    setclienList([...res.data.list])
  }

  useEffect(() => {
    getClient()
  }, [])

  useDidShow(() => {
    // 获取选择的客户
    const pages = Taro.getCurrentPages()
    const currPage = pages[pages.length - 1] // 获取当前页面
    // 判断是否有跳转选择客户
    if (currPage.data?.clientId && currPage.data?.clientId !== '') {
      setclientObj({
        clientId: currPage.data?.clientId,
        clientName: currPage.data?.clientName,
      })
      ShopCartRef.current.setSearchObj(val => ({ ...val, purchaser_id: currPage.data?.clientId }))
    }
    // 默认客户
    if (currPage.data?.clientId == null) {
      setclientObj({
        clientId: clienList.length > 0 ? clienList[0]?.id : -1,
        clientName: clienList.length > 0 ? clienList[0]?.name : '',
      })
    }
  })

  const [showShopCart, setShowShopCart] = useState(false)

  const [goodObj, setGoodsobj] = useState({})
  const ShopCartRef = useRef<any>()
  // 点击对应商品显示购物车
  const showCart = async(item) => {
    ShopCartRef.current.setSearchObj(e => ({ ...e, goodsId: item.id, purchaser_id: clientObj?.clientId, code_or_name: '' }))
    ShopCartRef.current.SearchRef.current.setInputCon('')
    setShowShopCart(true)
    setGoodsobj(item)
  }

  // 关闭弹窗
  const closePoup = () => {
    setShowShopCart(false)
  }

  // 获取关键字数据
  const [histroyList, setHistroyList] = useState<any[]>([])
  const { fetchData: historyFetch } = MpSearchHistoryList()
  const getHistory = async() => {
    Taro.showLoading({
      title: '加载中...',
      mask: true,
    })
    const res = await historyFetch()
    if (res.data) {
      setHistroyList([...res?.data?.list])
      Taro.hideLoading()
    }
  }

  // 搜索商品的数组
  const [searchList, setSearchList] = useState<any[]>([])
  // 是否有值输入框
  const [hasFonts, setHasFonts] = useState(false)
  useEffect(() => {
    getHistory()
  }, [])

  // 搜索获取商品数据
  const { fetchData: productFetch } = MpProductList()
  const { fetchData: historyputFetch } = MpSearchHistory()
  const getProduct = debounce(async(e) => {
    Taro.showLoading({
      title: '加载中...',
      mask: true,
    })
    await historyputFetch({ key: e, scene: 0 }).then((res) => {
      if (res.data) {
        getHistory()
      }
    })
    productFetch({ code_or_name: e }).then((res) => {
      if (res.data) {
        Taro.hideLoading()
        setSearchList([...res?.data?.list])
      }
    })
  }, 300)

  // 输入了搜索关键字
  const getSearchData = useCallback((e) => {
    if (!e) {
      setHasFonts(false)
      setSearchList([])
    }
    setdefaultvalue(e)
  }, [])

  // 搜索
  const onSearch = () => {
    if (defaultvalue) {
      setHasFonts(true)
      getProduct(defaultvalue)
    }
  }

  // 选择中后的值到输入框
  const [defaultvalue, setdefaultvalue] = useState('')

  // 点击关键字搜索内容
  const handItem = (item) => {
    Taro.showLoading({
      title: '加载中...',
      mask: true,
    })
    productFetch({ code_or_name: item.search_key }).then((res) => {
      if (res.data) {
        setHasFonts(true)
        setSearchList([...res.data.list])
        Taro.hideLoading()
      }
    })
    setdefaultvalue(item.search_key)
  }
  // 点击返回文字
  const handBack = () => {
    setHasFonts(false)
    setSearchList([])
  }

  // 删除历史搜索
  const { fetchData: deleteHistoryApi } = DeleteSearchHistoryList()
  const handDelete = async() => {
    Taro.showModal({
      title: '确定删除历史搜索记录吗？',
      async success(res) {
        if (res.confirm) {
          const result = await deleteHistoryApi()
          if (result.success) {
            Taro.showLoading({
              title: '请稍等...',
            })
            alert.success('删除成功')
            getHistory()
            Taro.hideLoading()
          }
          else {
            alert.error('删除失败')
            Taro.hideLoading()
          }
        }
        else if (res.cancel) {
          console.log('用户点击取消')
        }
      },
    })
  }

  return (
    <View className={styles.main}>
      <View className={styles.topBox}>
        <Search placeholder="请输入搜索布料" defaultValue={defaultvalue} showBtn clickOnSearch={onSearch} changeOnSearch={getSearchData} >
        </Search>
      </View>
      <View className={styles.line}></View>
      {
        !hasFonts && <>
          <View className={styles.topBox}>
            <View className={styles.histroyFonts}>历史搜索</View>
            {
              histroyList.length > 0
                && <View onClick={() => handDelete()}><IconFont name="icon-sousuoshanchu" size={40}></IconFont></View>
            }
          </View>
          {!!histroyList.length
                        && <View className={styles.bigBox}>
                          {histroyList.map((item, index) => {
                            return (
                              <View onClick={() => handItem(item)} key={index} className={styles.itemBox}>{item.search_key}</View>
                            )
                          })}
                        </View>
          }
          {
            !histroyList.length && <Empty picUrl="https://cdn.zzfzyc.com/empty.png" text="无搜索历史"></Empty>
          }
        </>
      }
      {
        hasFonts && <>
          <View className={styles.topBox}>
            <View className={styles.histroyLeft}>
              <View className={styles.histroyFonts}>搜索结果</View>
              <View className={styles.back} onClick={() => handBack()}>返回</View>
            </View>
          </View>
          {
            !!searchList.length && searchList.map((item, index) => {
              return (
                <Goods clickItem={() => showCart(item)} key={index} data={item}></Goods>
              )
            })
          }
          {
            !searchList.length && <Empty picUrl="https://cdn.zzfzyc.com/empty.png" text="暂无数据"></Empty>
          }
        </>
      }
      <ShopCart
        ref={ShopCartRef}
        clientName={clientObj?.clientName}
        clientId={clientObj?.clientId}
        obj={goodObj}
        showPopup={showShopCart}
        closePopup={closePoup}
      ></ShopCart>

    </View>

  )
}
export default memo(SearchPage)
// oninputEvent={(e, item) => { onInputEven?.(e, item) }}
