import { Image, Text, View } from '@tarojs/components'
import type { FC } from 'react'
import { forwardRef, memo, useCallback, useEffect, useRef, useState } from 'react'
import classnames from 'classnames'
import type { ShoppingStateContextValue } from '../../context'
import { Goods, ShoppingDispatchType, useShoppingDispatch, useShoppingState } from '../../context'
import type { ShoppingStore } from '../../context/shoppingStore'
import { events } from '../../context/shoppingStore'
import styles from './index.module.scss'
import MCheckbox from '@/components/checkbox'
import Counter from '@/components/counter'
import { debounce } from '@/common/util'
import { formatImgUrl, formatMeterDiv, formatMeterMul, formatPriceDiv } from '@/common/format'
import { EnumSaleMode } from '@/common/Enumerate'
import { AdjestShoppingCartApi } from '@/api/shopping/index'
import LabAndImg from '@/components/LabAndImg'
import { usePropsValue } from '@/use/useCommon'
import { selectList } from '@/common/Enumerate/BaseEnum'

interface PropsType {
  // state?: {
  //   Observer: ShoppingStore
  // }
  purchaserId: number
  itemData: Record<string, any> & object
  checked?: boolean
  // onChange?: (isChecked: boolean, goodsId: number) => void
}

const ColorKindItem: FC<PropsType> = (props) => {
  console.log('rerender component ColorKindItem', props)
  const { purchaserId, itemData, checked } = props

  const [isChecked, setCheck] = usePropsValue({
    defaultValue: checked,
    value: checked,
    // onChange: (value) => {
    //   setCheck(value)
    // },
  })

  const currentCount = useRef({
    roll: itemData.roll,
    length: itemData.length,
  })
  // 格式化数量
  const formatCount = (itemData) => {
    return itemData.sale_mode == EnumSaleMode.Bulk ? itemData.roll : formatMeterDiv(itemData.length)
  }

  const [count, setCount] = useState(formatCount(itemData))

  // 格式化金额
  const formatPrice = (price: number) => {
    return Number(formatPriceDiv(price))
  }

  const handleCountChange = (nextValue: number) => {
    setCount(nextValue)
  }

  // 格式化单位
  const formatUnit = (itemData) => {
    return itemData.sale_mode == EnumSaleMode.Bulk ? '条' : '米'
  }
  // 选中
  const handleSelect = () => {
    console.log('currentCount==>', currentCount.current, itemData)
    const payload = {
      id: itemData?.id,
      estimate_amount: itemData.estimate_amount,
      product_code: itemData.product_code,
      product_color_code: itemData.product_color_code,
      sale_mode: itemData.sale_mode,
      count: itemData.sale_mode === EnumSaleMode.Bulk ? currentCount.current.roll : Number(formatMeterDiv(currentCount.current.length)),
    }
    events.trigger('updatePurchaserMultipleSelection', purchaserId, payload, 'add', itemData?.id)
    setCheck(true)
  }
  // 未选中
  const handleClose = () => {
    events.trigger('updatePurchaserMultipleSelection', purchaserId, null, 'delete', itemData.id)
    setCheck(false)
  }
  const { fetchData } = AdjestShoppingCartApi()

  // 调整条数/米数的接口  并在调整完成后重新请求整个购物车页面
  const getInputValue = useCallback(debounce(async(num) => {
    console.log('num===>', num)
    if (num < selectList[itemData.sale_mode].minNum) {
      return
    }
    const targetColor = {
      roll: 0,
      length: 0,
    }
    if (itemData.sale_mode === EnumSaleMode.Bulk) {
      targetColor.roll = Math.trunc(num)
    }
    else {
      targetColor.length = formatMeterMul(num)
    }
    const res = await fetchData({
      id: itemData.id,
      roll: targetColor.roll,
      length: targetColor.length,
    })
    if (res.success) {
      currentCount.current = targetColor
      events.trigger('FetchSpecificPurchaser', purchaserId)
    }
    else {
      // 请求失败，赋值回初始数量
      setCount(formatCount(itemData))
    }
  }, 300), [itemData])

  const labAndImgObj = useCallback((item) => {
    return { lab: item.lab, rgb: item.rgb, texture_url: item.product_color_texture_url }
  }, [])

  useEffect(() => {
    console.log('itemData==>', itemData)
    setCount(formatCount(itemData))
  }, [itemData.roll, itemData.length])

  return (
    <MCheckbox
      status={isChecked}
      onSelect={handleSelect}
      onClose={handleClose}
      customClassName={classnames(styles.checkbox, isChecked ? styles.selected : '')}
      customTextClass={styles.colorKindItem}
    >
      <View className={styles.colorKindItem__left}>
        <LabAndImg suffix="!w200" value={labAndImgObj(itemData)} />
      </View>
      <View className={styles.colorKindItem__center}>
        <Text className={styles['colorKindItem__center--title']}>
          {itemData.product_code}# {itemData.product_name}
        </Text>
        <View className={styles['colorKindItem__center--detail']}>
          <Text className={styles['colorKindItem__center--ID']}>{itemData.product_color_code}</Text>
          <Text className={styles['colorKindItem__center--description']}>{itemData.product_color_name}</Text>
        </View>
      </View>
      <View className={styles.colorKindItem__right}>
        <View className={styles['colorKindItem__right--price']}>￥ {formatPrice(itemData.sale_price)}/{itemData.sale_mode == EnumSaleMode.Plate ? '米' : 'kg'}</View>
        {
          itemData.sale_mode === EnumSaleMode.Bulk && itemData?.standard_sale_price !== itemData?.sale_price && <View className={styles['colorKindItem__right--yuanPrice']}>￥ {formatPrice(itemData?.standard_sale_price)}/{itemData.sale_mode == EnumSaleMode.Plate ? '米' : 'kg'}</View>
        }
        <View className={styles['colorKindItem__right--counter']}>
          <Counter
            onBlue={getInputValue}
            defaultNum={count}
            onChange={handleCountChange}
            step={itemData.step}
            digits={itemData.digits}
            onClickBtn={getInputValue}
            unit={formatUnit(itemData)}
            minNum={selectList[itemData.sale_mode].minNum}
            maxNum={selectList[itemData.sale_mode].maxNum}
          />
        </View>
      </View>
    </MCheckbox>
  )
}

export default memo(ColorKindItem)
