import type { FC, ReactNode } from 'react'
import { useEffect, useMemo, useReducer, useRef } from 'react'
import type {
  ColorStore,
  ShoppingAction,
  ShoppingStateContextValue,
} from '../../context'
import {
  ShoppingDispatchContext,
  ShoppingDispatchType,
  ShoppingStateContext,
  shoppingReducer, throwError,
} from '../../context'
import { ShoppingStore } from '../../context/shoppingStore'

export interface ShoppingCartPropsType {
  initialValues?: ColorStore
  children?: ReactNode
}
// 购物车上下文组件
export const ShoppingProvider: FC<ShoppingCartPropsType> = (props) => {
  const { children, initialValues } = props

  // const onTriggerCheckboxRef = useRef(onTriggerCheckbox)
  // onTriggerCheckboxRef.current = onTriggerCheckbox
  // 发布订阅
  const Observer = useMemo(() => new ShoppingStore(), [])

  const [state, dispatch] = useReducer<(state: ShoppingStateContextValue, action: ShoppingAction) => ShoppingStateContextValue>(shoppingReducer, {
    colorStore: initialValues || {},
    currentCheckedPurchaserId: -1,
    isManageStatus: false,
    isMultipleSelection: false,
    selectedAmount: 0,
    Observer,
  })

  return (
    <ShoppingStateContext.Provider value={state}>
      <ShoppingDispatchContext.Provider value={dispatch}>{children}</ShoppingDispatchContext.Provider>
    </ShoppingStateContext.Provider>
  )
}
