import { Text, View } from '@tarojs/components'
import Taro from '@tarojs/taro'
import type { FC } from 'react'
import { forwardRef, memo, useEffect, useMemo, useRef, useState } from 'react'
import classnames from 'classnames'
import ColorKindItem from '../colorKindItem'
import type { Goods, GoodsMeta, ShoppingStateContextValue } from '../../context'
import { ShoppingDispatchType, useShoppingDispatch, useShoppingState } from '../../context'
import { events } from '../../context/shoppingStore'
import styles from './index.module.scss'
import { formatMeterDiv, formatPriceDiv } from '@/common/format'
import LayoutBlock from '@/components/layoutBlock'
import Tag from '@/components/tag'
import Divider from '@/components/Divider'
import { EnumSaleMode } from '@/common/Enumerate'
import IconFont from '@/components/iconfont/iconfont'
import { alert, goLink, isEmptyObject } from '@/common/common'
import { ShoppingCartDeleteApi, ShoppingCartListApi } from '@/api'
import { usePropsValue } from '@/use/useCommon'
import { selectList } from '@/common/Enumerate/BaseEnum'

export enum BackEndSaleModeListFieldMap {
  bulk_color_list = 0,
  length_cut_color_list = 1,
  weight_cut_color_list = 2,
}
export const BackEndSaleModeListField: BackEndSaleModeListFieldMap[] = [
  BackEndSaleModeListFieldMap.bulk_color_list,
  BackEndSaleModeListFieldMap.length_cut_color_list,
  BackEndSaleModeListFieldMap.weight_cut_color_list,
]
interface ButtonPropsType {
  isActive: boolean
  onClick?: Function
  children?: React.ReactNode
  customStyle?: React.CSSProperties
}
type OperationType = 'add' | 'delete'
// 订单类型
const SaleModeButton: FC<ButtonPropsType> = (props) => {
  const { onClick, children, isActive = false, customStyle } = props
  const handleClick = () => {
    onClick?.()
  }
  return (
    <View className={classnames(styles.saleModeButton, isActive && styles['saleModeButton--active'])} style={customStyle} onClick={handleClick}>
      <View className={classnames(styles['saleModeButton--text'])}>{children}</View>
    </View>
  )
}

const DrawerButton = ({ isOpen }: { isOpen: boolean }) => {
  return (
    <View className={styles.drawerButton}>
      {isOpen ? <IconFont name="icon-shouqi" size={52}></IconFont> : <IconFont name="icon-zhankai" size={52}></IconFont>}
    </View>
  )
}
const DrawerButtonWithMemo = memo(DrawerButton)

const ColorKindItemContainer = (props: { id: number; item: any; purchaser_id: number; checked: boolean }) => {
  const { id, item, purchaser_id, checked } = props
  return <ColorKindItem checked={checked} purchaserId={purchaser_id} key={id} itemData={item}></ColorKindItem>
}
const ColorKindItemContainerWithMemo = memo(ColorKindItemContainer)

interface GoodsListPropType {
  itemData?: ShoppingCartData
  multipleSelection?: GoodsMeta['multipleSelection']
  selected: EnumSaleMode
}
const GoodsList = (props: GoodsListPropType) => {
  const { itemData, selected, multipleSelection } = props
  console.log('rerender component GoodsList', props, props.itemData?.purchaser_name)

  return (
    <>
      {itemData?.[BackEndSaleModeListFieldMap[selected]].length !== 0
        ? (
          itemData?.[BackEndSaleModeListFieldMap[selected]].map((item) => {
            console.log('item===>', item, multipleSelection?.hasOwnProperty(item.id))
            return <ColorKindItemContainerWithMemo checked={multipleSelection?.hasOwnProperty(item.id) || false} key={item.id} id={item.id} purchaser_id={itemData.purchaser_id} item={item} />
          })
        )
        : (
          <View className={styles.noList}>暂无数据</View>
        )}
    </>
  )
}
const GoodsListWithMemo = memo<GoodsListPropType>(GoodsList)

interface PropsType {
  itemData?: ShoppingCartData
  onDeleteItem: () => void
  state?: {
    // colorStore?: ColorStore
    // multipleSelection?: GoodsMeta['multipleSelection']
    currentCheckedPurchaserId?: number
    isMultipleSelection?: ShoppingStateContextValue['isMultipleSelection']
  }
}

const ShoppingCartItem = (props: PropsType) => {
  const { state } = props
  console.log('rerender component ShoppingCartItem', props, props.itemData?.purchaser_name)

  const currentItemData = useRef<ShoppingCartData | undefined>(props.itemData)
  // 及时更新 useRef 的数据 不然拿到的还是旧数据
  currentItemData.current = props.itemData

  const [itemData, setItemData] = usePropsValue({
    value: currentItemData.current,
    defaultValue: props.itemData,
  })

  const [multipleSelection, setMultipleSelection] = useState<GoodsMeta['goodsKind']>([])
  console.log('rerender multipleSelection', multipleSelection, props.itemData?.purchaser_name)

  const { currentCheckedPurchaserId, isMultipleSelection } = state!

  const dispatch = useShoppingDispatch()

  const [openDetail, setOpenDetail] = useState(false)

  const [selected, setSelect] = useState<EnumSaleMode>(0)

  useEffect(() => {
    if (currentCheckedPurchaserId === itemData?.purchaser_id) {
      console.log('isMultipleSelection', isMultipleSelection, currentCheckedPurchaserId, itemData?.purchaser_id)
      if (isMultipleSelection) {
        setMultipleSelection(Object.fromEntries(
          itemData?.[BackEndSaleModeListFieldMap[selected]].map(item => [
            item?.id,
            {
              id: item?.id,
              estimate_amount: item.estimate_amount,
              product_code: item.product_code,
              product_color_code: item.product_color_code,
              sale_mode: item.sale_mode,
              count: selected === EnumSaleMode.Bulk ? item.roll : Number(formatMeterDiv(item.length)),
            },
          ]),
        ))
      }
    }
  }, [isMultipleSelection, itemData, selected, currentCheckedPurchaserId])

  const cleanMultiSelection = () => {
    if (currentCheckedPurchaserId === itemData?.purchaser_id) {
      setMultipleSelection({})
    }
  }

  // const [isPending, startTransition] = useTransition()
  const onSelectOrderType = (type: EnumSaleMode) => {
    // if (isPending) { return }
    if (selected !== type) {
      cleanMultiSelection()
    }
    setSelect(type)
    // 重置预估金额
    dispatch({ type: ShoppingDispatchType.UPDATE_SELECTED_AMOUNT, data: 0 })
    dispatch({ type: ShoppingDispatchType.UPDATE_MULTIPLE_SELECTION_STATUS, data: false })
  }

  const handleClickLayout = () => {
    if (currentCheckedPurchaserId === props.itemData?.purchaser_id) {
      return
    }
    currentItemData.current = props.itemData
    setItemData(props.itemData)
    dispatch({ type: ShoppingDispatchType.UPDATE_SELECTED_AMOUNT, data: 0 })
    dispatch({
      type: ShoppingDispatchType.UPDATE_CURRENT_CHECKED_PURCHASERID,
      data: props.itemData?.purchaser_id as number,
    })
    dispatch({ type: ShoppingDispatchType.UPDATE_MULTIPLE_SELECTION_STATUS, data: Object.values(multipleSelection!).length === itemData?.[BackEndSaleModeListFieldMap[selected]].length })
  }
  const handleOpenDetail = () => {
    setOpenDetail(isOpen => !isOpen)
    handleClickLayout()
  }
  // 统计已选面料
  const materialChecked = useMemo(() => {
    // const multipleSelection = colorStore?.[itemData!.purchaser_id]?.multipleSelection
    console.log('materialChecked', multipleSelection)
    if (!multipleSelection || isEmptyObject(multipleSelection)) { return 0 }
    return new Set(
      Object.values(multipleSelection)?.reduce((prev, item: Goods) => {
        if (item.sale_mode === selected) {
          return [...prev, item.product_code]
        }
        return prev
      }, []),
    ).size
  }, [multipleSelection, currentCheckedPurchaserId, selected])

  // 统计已选颜色
  const colorChecked = useMemo(() => {
    // const multipleSelection = colorStore?.[itemData!.purchaser_id]?.multipleSelection
    console.log('colorChecked item', multipleSelection)
    if (!multipleSelection || isEmptyObject(multipleSelection)) { return 0 }
    return new Set(
      Object.values(multipleSelection).reduce((prev, item: Goods) => {
        if (item.sale_mode === selected) {
          return [...prev, item.product_color_code]
        }
        return prev
      }, []),
    ).size
  }, [multipleSelection, currentCheckedPurchaserId, selected])

  // 统计已选条数 / 米数
  const lengthOrRollChecked = useMemo(() => {
    // const multipleSelection = colorStore?.[itemData!.purchaser_id]?.multipleSelection
    console.log('lengthOrRollChecked item', multipleSelection, currentCheckedPurchaserId)
    if (!multipleSelection || isEmptyObject(multipleSelection)) { return 0 }
    return (
      Object.values(multipleSelection).reduce((prev, item: Goods) => {
        return prev + item.count
      }, 0) || 0
    )
  }, [multipleSelection, currentCheckedPurchaserId, selected])

  const { fetchData } = ShoppingCartListApi()

  // 更新当前客户的多选项
  const updatePurchaserMultipleSelection = (purchaserId: number, payload: Goods, operationType: OperationType, goodsId: number) => {
    if (itemData?.purchaser_id === purchaserId) {
      console.log('updatePurchaserMultipleSelection', purchaserId)
      if (operationType === 'add') {
        const nextMultipleSelection = {
          ...multipleSelection,
          [goodsId]: payload,
        }

        console.log('nextMultipleSelection', nextMultipleSelection)
        setMultipleSelection(nextMultipleSelection)
        // 全选标识
        if (itemData?.[BackEndSaleModeListFieldMap[selected]].length === Object.keys(nextMultipleSelection).length) {
          dispatch({ type: ShoppingDispatchType.UPDATE_MULTIPLE_SELECTION_STATUS, data: true })
        }
      }
      else if (operationType === 'delete') {
        const temp = { ...multipleSelection }
        delete temp?.[goodsId]

        setMultipleSelection(temp)
        if (isMultipleSelection) {
          dispatch({ type: ShoppingDispatchType.UPDATE_MULTIPLE_SELECTION_STATUS, data: false })
        }
      }
    }
  }

  const handleOtherHighLight = (purchaserId: number, deletedItemData: any) => {
    if (purchaserId === itemData?.purchaser_id) {
      // 当前订单类型还有数据，跳过
      if (deletedItemData?.[BackEndSaleModeListFieldMap[selected]].length !== 0) { return }
      // 当前订单类型没有数据，高亮其他订单类型
      for (let i = 0; i < BackEndSaleModeListField.length; i++) {
        if (deletedItemData?.[BackEndSaleModeListFieldMap[i]].length !== 0) {
          // 控制高亮
          onSelectOrderType(i)
          return
        }
      }
    }
  }

  useEffect(() => {
    events.on('handleOtherHighLight', handleOtherHighLight)
    return () => {
      events.off('handleOtherHighLight', handleOtherHighLight)
    }
  }, [handleOtherHighLight])

  // 发布订阅
  useEffect(() => {
    events.on('updatePurchaserMultipleSelection', updatePurchaserMultipleSelection)
    return () => {
      events.off('updatePurchaserMultipleSelection', updatePurchaserMultipleSelection)
    }
  }, [updatePurchaserMultipleSelection])

  const fetchSpecificPurchaser = async(id: number) => {
    if (itemData?.purchaser_id !== id) { return }
    console.log('request new data start run')
    const res = await fetchData({
      purchaser_id: id,
    })
    console.log('res===>', res)
    if (res.success) {
      // 转成 对象形式
      // 4562: {
      //  id: 4562,
      //    ...
      // }
      if (!res.data.list.length) { return alert.error('修改数量失败') }

      const newGoodsKind = Object.fromEntries(
        res.data.list[0]?.[BackEndSaleModeListFieldMap[selected]].map(item => [
          item?.id,
          {
            id: item?.id,
            estimate_amount: item.estimate_amount,
            product_code: item.product_code,
            product_color_code: item.product_color_code,
            sale_mode: item.sale_mode,
            count: selected === EnumSaleMode.Bulk ? item.roll : Number(formatMeterDiv(item.length)),
          },
        ]),
      )

      setMultipleSelection(prev => Object.fromEntries(
        Object.keys(prev!)
          .map(id => [
            id,
            {
              id: Number(id),
              estimate_amount: newGoodsKind[id].estimate_amount,
              product_code: newGoodsKind[id].product_code,
              product_color_code: newGoodsKind[id].product_color_code,
              sale_mode: newGoodsKind[id].sale_mode,
              count: newGoodsKind[id].count,
            },
          ]),
      ))
      currentItemData.current = res.data.list[0]
      setItemData(() => res.data.list[0])
    }
    else {
      alert.error('请求失败')
    }
  }

  const { fetchData: deleteApi } = ShoppingCartDeleteApi()
  // 批量某个客户的删除商品
  const handleDelete = async(id: number) => {
    console.log('handleDelete', itemData?.purchaser_id, id)
    if (id !== itemData?.purchaser_id) { return }
    console.log('multipleSelection', multipleSelection)
    const checked: Goods[] = Object.values(multipleSelection!)
    if (checked.length === 0) {
      return alert.error('请选择商品')
    }
    console.log('checked==>', checked)
    Taro.showModal({
      title: '要删除这些商品吗？',
      async success(res) {
        if (res.confirm) {
          alert.showLoading('删除中')
          const checkedIds = checked.map(item => item.id)
          const res = await deleteApi({
            ids: checkedIds.join(','),
          })
          if (res.success) {
            alert.hideLoading()
            alert.success('删除成功')
            cleanMultiSelection()
            // const deleted = await fetchData(searchOptions)
            dispatch({ type: ShoppingDispatchType.UPDATE_MULTIPLE_SELECTION_STATUS, data: false })

            // handleOtherHighLight(currentCheckedPurchaserId!, deleted.data.list.find(item => item.purchaser_id === currentCheckedPurchaserId))
            props?.onDeleteItem()
            // Observer.notify(currentCheckedPurchaserId)
          }
          else {
            alert.hideLoading()
            alert.none(res.msg)
          }
        }
      },
    })
  }
  const handleSettleAccount = (id: number) => {
    console.log('handleSettleAccount', itemData?.purchaser_id, id)
    if (id !== itemData?.purchaser_id) { return }
    if (!multipleSelection) { return alert.error('请先选择客户') }
    const checkedGoodsKind = Object.values(multipleSelection).reduce((prev, item: Goods) => {
      return [...prev, item.id]
    }, [])
    if (checkedGoodsKind.length === 0) { return alert.error('请先选择商品') }
    goLink('/pages/submitOrder/index', {
      purchaser_id: currentCheckedPurchaserId,
      sale_mode: selected,
      shopping_cart_product_color_list: JSON.stringify(checkedGoodsKind),
      purchaser_name: itemData?.purchaser_name,
      isBuyNew: false,
    })
  }

  // 发布订阅
  useEffect(() => {
    events.on('FetchSpecificPurchaser', fetchSpecificPurchaser)
    events.on('DeleteSpecificItem', handleDelete)
    events.on('SettleAccount', handleSettleAccount)
    events.on('CleanMultiSelection', cleanMultiSelection)
    // 取消订阅
    return () => {
      events.off('FetchSpecificPurchaser', fetchSpecificPurchaser)
      events.off('DeleteSpecificItem', handleDelete)
      events.off('SettleAccount', handleSettleAccount)
      events.off('CleanMultiSelection', cleanMultiSelection)
    }
  }, [fetchSpecificPurchaser, handleDelete, handleSettleAccount])

  useEffect(() => {
    if (itemData?.purchaser_id === currentCheckedPurchaserId) {
      const multipleSelectionArr = Object.values(multipleSelection!)
      console.log('shoppingCartItem multipleSelection change', multipleSelection, itemData?.purchaser_name)
      if (multipleSelectionArr.length > 0) {
        const result = multipleSelectionArr.reduce((prev: number, value: Goods) => {
          return prev + Number(formatPriceDiv(value.estimate_amount))
        }, 0) as number
        // 同步修改上下文的 预估金额
        dispatch({ type: ShoppingDispatchType.UPDATE_SELECTED_AMOUNT, data: result })
      }
      else {
        dispatch({ type: ShoppingDispatchType.UPDATE_SELECTED_AMOUNT, data: 0 })
      }
    }
  }, [multipleSelection])

  return (
    <LayoutBlock
      circle
      customClassName={classnames(styles.layout, itemData?.purchaser_id === currentCheckedPurchaserId ? styles.selected : '')}
      customStyle={{ paddingBottom: !openDetail ? '24rpx' : 0 }}
      onClick={handleClickLayout}
    >
      <View className={classnames(styles.checkbox, 'flex-row', 'justify-between')} onClick={handleOpenDetail}>
        <View className={styles.topItem}>
          <View className="flex-row items-center">
            <View className={styles.topTitle}>{itemData?.purchaser_name}</View>
            {
              itemData?.sale_user_name && <Tag type="info" size="normal" circle customStyle={{ backgroundColor: '#f0f0f0', borderColor: 'transparent' }} plain>
                {itemData?.sale_user_name}
              </Tag>
            }
          </View>
          <View className={styles.summary}>
            <Text>
              已选 {materialChecked} 种面料，{colorChecked} 个颜色，共{' '}
              {selected === EnumSaleMode.Bulk ? `${lengthOrRollChecked} 条` : `${(lengthOrRollChecked).toLocaleString()} 米`}
            </Text>
          </View>
        </View>
        <DrawerButtonWithMemo isOpen={openDetail} />
      </View>
      {/* 减少节点渲染 */}
      {openDetail && (
        <>
          <View className={classnames(styles.detailBox, openDetail ? styles.drawerOpen : styles.drawerClose)}>
            {/* <View className={styles.checkbox}>
              <Divider customClassName={styles.line}></Divider>
            </View> */}
            <View className={styles.orderType}>
              {/* <View className={styles.orderTitle}>订单类型</View> */}
              <View className={styles.orderTypeDetail}>
                <SaleModeButton
                  customStyle={{ padding: '0 50rpx', height: '60rpx' }}
                  isActive={selected === EnumSaleMode.Bulk}
                  onClick={() => onSelectOrderType(EnumSaleMode.Bulk)}
                >
                  大货
                </SaleModeButton>
                <SaleModeButton
                  customStyle={{ padding: '0 50rpx', height: '60rpx' }}
                  isActive={selected === EnumSaleMode.Plate}
                  onClick={() => onSelectOrderType(EnumSaleMode.Plate)}
                >
                  剪板
                </SaleModeButton>
                <SaleModeButton
                  customStyle={{ padding: '0 50rpx', height: '60rpx' }}
                  isActive={selected === EnumSaleMode.BulkCut}
                  onClick={() => onSelectOrderType(EnumSaleMode.BulkCut)}
                >
                  散剪
                </SaleModeButton>
              </View>
            </View>
            <View className={styles.orderColor}>
              <Divider customClassName={styles.line}></Divider>
              <View className={classnames(styles.orderTitle, 'justify-between')}>
                <Text>布料颜色</Text>
                <Tag type="danger" size="normal" circle plain>
                  {selectList[selected].title}单位：{selectList[selected].unit}
                </Tag>
              </View>
            </View>
            <View className={styles.orderContainer}>
              <GoodsListWithMemo
                multipleSelection={multipleSelection}
                itemData={itemData}
                selected={selected}
                // isPending={isPending}
                // startTransition={startTransition}
                // onSelectOrderType={onSelectOrderType}
              />
            </View>
          </View>
        </>
      )}
    </LayoutBlock>
  )
}

// State 分割组件 思路就是把 context 直接通过 props 的形式传给组件，这样的话就解决了 context 强制刷新 memo 的问题了
// 那么当 context 内的 value 被更新的时候，react 只会强制渲染 Wrapper
const withStateSlice = (comp, slice: (state: ShoppingStateContextValue, props: PropsType) => any) => {
  const MemoComp = memo(comp)
  // const MemoComp = memo(comp, (prevProps, nextProps) => {
  //   let needMemo = true
  //   if (prevProps.itemData !== nextProps.itemData) {
  //     console.log('itemData 有变化')
  //     needMemo = false
  //   }
  //   if (prevProps.state !== nextProps.state) {
  //     console.log('state 有变化')
  //     needMemo = false
  //   }
  //   // if (JSON.stringify(prevProps.itemData) !== JSON.stringify(nextProps.itemData)) {
  //   //   console.log('itemData 有变化')
  //   //   needMemo = false
  //   // }
  //   if (prevProps.state.currentCheckedPurchaserId !== nextProps.state.currentCheckedPurchaserId) {
  //     console.log('currentCheckedPurchaserId 有变化')
  //     needMemo = false
  //   }
  //   if (prevProps.state.isMultipleSelection !== nextProps.state.isMultipleSelection) {
  //     console.log('isMultipleSelection 有变化')
  //     needMemo = false
  //   }
  //   if (prevProps.itemData.purchaser_name === '孜然烤鱿鱼') {
  //     console.log('更新 孜然烤鱿鱼', !needMemo)
  //     console.log('------withStateSlice props-------')
  //     console.log('withStateSlice props prevProps', prevProps)
  //     console.log('withStateSlice props prevProps comparison itemData', prevProps.itemData === nextProps.itemData)
  //     console.log('withStateSlice props nextProps', nextProps)
  //     console.log('withStateSlice props prevProps comparison currentCheckedPurchaserId', prevProps.state.currentCheckedPurchaserId === nextProps.state.currentCheckedPurchaserId)
  //     console.log('withStateSlice props prevProps comparison state', prevProps.state === nextProps.state)
  //     console.log('------withStateSlice props-------')
  //   }
  //   return needMemo
  // })
  const Wrapper = (props: PropsType, ref) => {
    const state = useShoppingState()
    // console.log('Wrapper colorStore', state.colorStore)
    const sliced = useMemo(() => {
      return {
        currentCheckedPurchaserId: state.currentCheckedPurchaserId,
        isMultipleSelection: state.isMultipleSelection,
      }
    }, [state.currentCheckedPurchaserId, state.isMultipleSelection])
    return <MemoComp ref={ref} state={sliced} {...props} />
  }
  return memo(forwardRef(Wrapper))
}

const ShoppingCartItemWithStateSlice = withStateSlice(ShoppingCartItem, (state: ShoppingStateContextValue) => ({
  // colorStore: state.colorStore,
  // multipleSelection: state.colorStore?.[props.itemData!.purchaser_id]?.multipleSelection,
  currentCheckedPurchaserId: state.currentCheckedPurchaserId,
  isMultipleSelection: state.isMultipleSelection,
}))

export default ShoppingCartItemWithStateSlice
